@@ -105,6 +105,7 @@
 #define SYSDB_UUID "uniqueID"
 #define SYSDB_SID "objectSID"
 #define SYSDB_PRIMARY_GROUP "ADPrimaryGroupID"
+#define SYSDB_PRIMARY_GROUP_GIDNUM "origPrimaryGroupGidNumber"
 #define SYSDB_SID_STR "objectSIDString"
 #define SYSDB_UPN "userPrincipalName"
 #define SYSDB_CCACHE_FILE "ccacheFile"
@@ -124,12 +125,15 @@
 #define SYSDB_SUBDOMAIN_REALM "realmName"
 #define SYSDB_SUBDOMAIN_FLAT "flatName"
 #define SYSDB_SUBDOMAIN_ID "domainID"
+#define SYSDB_SUBDOMAIN_MPG "mpg"
+#define SYSDB_SUBDOMAIN_ENUM "enumerate"
 
 #define SYSDB_BASE_ID "baseID"
 #define SYSDB_ID_RANGE_SIZE "idRangeSize"
 #define SYSDB_BASE_RID "baseRID"
 #define SYSDB_SECONDARY_BASE_RID "secondaryBaseRID"
 #define SYSDB_DOMAIN_ID "domainID"
+#define SYSDB_ID_RANGE_TYPE "idRangeType"
 
 #define SYSDB_NEXTID_FILTER "("SYSDB_NEXTID"=*)"
 
@@ -167,6 +171,8 @@
                         SYSDB_GIDNUM, SYSDB_GECOS, \
                         SYSDB_HOMEDIR, SYSDB_SHELL, \
                         SYSDB_DEFAULT_ATTRS, \
+                        SYSDB_PRIMARY_GROUP_GIDNUM, \
+                        SYSDB_SID_STR, \
                         NULL}
 #define SYSDB_GRSRC_ATTRS {SYSDB_NAME, SYSDB_GIDNUM, \
                            SYSDB_MEMBERUID, \
@@ -245,6 +251,7 @@ struct range_info {
     uint32_t base_rid;
     uint32_t secondary_base_rid;
     char *trusted_dom_sid;
+    char *range_type;
 };
 
 
@@ -364,15 +371,15 @@ errno_t sysdb_domain_create(struct sysdb_ctx *sysdb, const char *domain_name);
 
 errno_t sysdb_subdomain_store(struct sysdb_ctx *sysdb,
                               const char *name, const char *realm,
-                              const char *flat_name, const char *domain_id);
+                              const char *flat_name, const char *domain_id,
+                              bool mpg, bool enumerate);
 
 errno_t sysdb_update_subdomains(struct sss_domain_info *domain);
 
 errno_t sysdb_master_domain_update(struct sss_domain_info *domain);
 
 errno_t sysdb_master_domain_add_info(struct sss_domain_info *domain,
-                                     const char *realm, const char *flat,
-                                     const char *id);
+                                     const char *flat, const char *id);
 
 errno_t sysdb_subdomain_delete(struct sysdb_ctx *sysdb, const char *name);
 
@@ -608,7 +615,9 @@ int sysdb_add_incomplete_group(struct sysdb_ctx *sysdb,
                                struct sss_domain_info *domain,
                                const char *name,
                                gid_t gid,
-                               const char *original_dn, bool posix,
+                               const char *original_dn,
+                               const char *sid_str,
+                               bool posix,
                                time_t now);
 
 /* Add netgroup (only basic attrs and w/o checks) */
