@@ -33,9 +33,17 @@ errno_t sysdb_update_subdomains(struct sss_domain_info *domain)
                            SYSDB_SUBDOMAIN_REALM,
                            SYSDB_SUBDOMAIN_FLAT,
                            SYSDB_SUBDOMAIN_ID,
+                           SYSDB_SUBDOMAIN_MPG,
+                           SYSDB_SUBDOMAIN_ENUM,
                            NULL};
     struct sss_domain_info *dom;
     struct ldb_dn *basedn;
+    const char *name;
+    const char *realm;
+    const char *flat;
+    const char *id;
+    bool mpg;
+    bool enumerate;
 
     tmp_ctx = talloc_new(NULL);
     if (tmp_ctx == NULL) {
@@ -68,10 +76,6 @@ errno_t sysdb_update_subdomains(struct sss_domain_info *domain)
     }
 
     for (i = 0; i < res->count; i++) {
-        const char *name;
-        const char *realm;
-        const char *flat;
-        const char *id;
 
         name = ldb_msg_find_attr_as_string(res->msgs[i], "cn", NULL);
         if (name == NULL) {
@@ -91,6 +95,12 @@ errno_t sysdb_update_subdomains(struct sss_domain_info *domain)
         id = ldb_msg_find_attr_as_string(res->msgs[i],
                                          SYSDB_SUBDOMAIN_ID, NULL);
 
+        mpg = ldb_msg_find_attr_as_bool(res->msgs[i],
+                                        SYSDB_SUBDOMAIN_MPG, false);
+
+        enumerate = ldb_msg_find_attr_as_bool(res->msgs[i],
+                                              SYSDB_SUBDOMAIN_ENUM, false);
+
         /* explicitly use dom->next as we need to check 'disabled' domains */
         for (dom = domain->subdomains; dom; dom = dom->next) {
             if (strcasecmp(dom->name, name) == 0) {
@@ -129,12 +139,30 @@ errno_t sysdb_update_subdomains(struct sss_domain_info *domain)
                         goto done;
                     }
                 }
+
+                if (dom->mpg != mpg) {
+                    DEBUG(SSSDBG_TRACE_INTERNAL,
+                          ("MPG state change from [%s] to [%s]!\n",
+                           dom->mpg ? "true" : "false",
+                           mpg ? "true" : "false"));
+                    dom->mpg = mpg;
+                }
+
+                if (dom->enumerate != enumerate) {
+                    DEBUG(SSSDBG_TRACE_INTERNAL,
+                          ("MPG state change from [%s] to [%s]!\n",
+                           dom->enumerate ? "true" : "false",
+                           enumerate ? "true" : "false"));
+                    dom->enumerate = enumerate;
+                }
+
                 break;
             }
         }
         /* If not found in loop it is a new subdomain */
         if (dom == NULL) {
-            dom = new_subdomain(domain, domain, name, realm, flat, id);
+            dom = new_subdomain(domain, domain, name, realm,
+                                flat, id, mpg, enumerate);
             if (dom == NULL) {
                 ret = ENOMEM;
                 goto done;
@@ -237,8 +265,7 @@ done:
 }
 
 errno_t sysdb_master_domain_add_info(struct sss_domain_info *domain,
-                                     const char *realm, const char *flat,
-                                     const char *id)
+                                     const char *flat, const char *id)
 {
     TALLOC_CTX *tmp_ctx;
     struct ldb_message *msg;
@@ -250,11 +277,6 @@ errno_t sysdb_master_domain_add_info(struct sss_domain_info *domain,
         return ENOMEM;
     }
 
-    ret = sysdb_master_domain_update(domain);
-    if (ret != EOK) {
-        goto done;
-    }
-
     msg = ldb_msg_new(tmp_ctx);
     if (msg == NULL) {
         ret = ENOMEM;
@@ -268,24 +290,6 @@ errno_t sysdb_master_domain_add_info(struct sss_domain_info *domain,
         goto done;
     }
 
-    if (realm != NULL && (domain->realm == NULL ||
-                          strcmp(domain->realm, realm) != 0)) {
-        ret = ldb_msg_add_empty(msg, SYSDB_SUBDOMAIN_REALM,
-                                LDB_FLAG_MOD_REPLACE, NULL);
-        if (ret != LDB_SUCCESS) {
-            ret = sysdb_error_to_errno(ret);
-            goto done;
-        }
-
-        ret = ldb_msg_add_string(msg, SYSDB_SUBDOMAIN_REALM, realm);
-        if (ret != LDB_SUCCESS) {
-            ret = sysdb_error_to_errno(ret);
-            goto done;
-        }
-
-        do_update = true;
-    }
-
     if (flat != NULL && (domain->flat_name == NULL ||
                          strcmp(domain->flat_name, flat) != 0)) {
         ret = ldb_msg_add_empty(msg, SYSDB_SUBDOMAIN_FLAT,
@@ -336,6 +340,11 @@ errno_t sysdb_master_domain_add_info(struct sss_domain_info *domain,
         goto done;
     }
 
+    ret = sysdb_master_domain_update(domain);
+    if (ret != EOK) {
+        goto done;
+    }
+
     ret = EOK;
 
 done:
@@ -346,7 +355,8 @@ done:
 
 errno_t sysdb_subdomain_store(struct sysdb_ctx *sysdb,
                               const char *name, const char *realm,
-                              const char *flat_name, const char *domain_id)
+                              const char *flat_name, const char *domain_id,
+                              bool mpg, bool enumerate)
 {
     TALLOC_CTX *tmp_ctx;
     struct ldb_message *msg;
@@ -356,12 +366,17 @@ errno_t sysdb_subdomain_store(struct sysdb_ctx *sysdb,
                            SYSDB_SUBDOMAIN_REALM,
                            SYSDB_SUBDOMAIN_FLAT,
                            SYSDB_SUBDOMAIN_ID,
+                           SYSDB_SUBDOMAIN_MPG,
+                           SYSDB_SUBDOMAIN_ENUM,
                            NULL};
     const char *tmp_str;
+    bool tmp_bool;
     bool store = false;
     int realm_flags = 0;
     int flat_flags = 0;
     int id_flags = 0;
+    int mpg_flags = 0;
+    int enum_flags = 0;
     int ret;
 
     tmp_ctx = talloc_new(NULL);
@@ -390,6 +405,8 @@ errno_t sysdb_subdomain_store(struct sysdb_ctx *sysdb,
         if (realm) realm_flags = LDB_FLAG_MOD_ADD;
         if (flat_name) flat_flags = LDB_FLAG_MOD_ADD;
         if (domain_id) id_flags = LDB_FLAG_MOD_ADD;
+        mpg_flags = LDB_FLAG_MOD_ADD;
+        enum_flags = LDB_FLAG_MOD_ADD;
     } else if (res->count != 1) {
         ret = EINVAL;
         goto done;
@@ -415,9 +432,21 @@ errno_t sysdb_subdomain_store(struct sysdb_ctx *sysdb,
                 id_flags = LDB_FLAG_MOD_REPLACE;
             }
         }
+
+        tmp_bool = ldb_msg_find_attr_as_bool(res->msgs[0], SYSDB_SUBDOMAIN_MPG,
+                                             !mpg);
+        if (tmp_bool != mpg) {
+            mpg_flags = LDB_FLAG_MOD_REPLACE;
+        }
+        tmp_bool = ldb_msg_find_attr_as_bool(res->msgs[0], SYSDB_SUBDOMAIN_ENUM,
+                                             !enumerate);
+        if (tmp_bool != enumerate) {
+            enum_flags = LDB_FLAG_MOD_REPLACE;
+        }
     }
 
-    if (!store && realm_flags == 0 && flat_flags == 0 && id_flags == 0) {
+    if (!store && realm_flags == 0 && flat_flags == 0 && id_flags == 0
+            && mpg_flags == 0 && enum_flags == 0) {
         ret = EOK;
         goto done;
     }
@@ -429,7 +458,7 @@ errno_t sysdb_subdomain_store(struct sysdb_ctx *sysdb,
     }
     msg->dn = dn;
 
-   if (store) {
+    if (store) {
         ret = ldb_msg_add_empty(msg, SYSDB_OBJECTCLASS, LDB_FLAG_MOD_ADD, NULL);
         if (ret != LDB_SUCCESS) {
             ret = sysdb_error_to_errno(ret);
@@ -485,6 +514,36 @@ errno_t sysdb_subdomain_store(struct sysdb_ctx *sysdb,
         }
     }
 
+    if (mpg_flags) {
+        ret = ldb_msg_add_empty(msg, SYSDB_SUBDOMAIN_MPG, mpg_flags, NULL);
+        if (ret != LDB_SUCCESS) {
+            ret = sysdb_error_to_errno(ret);
+            goto done;
+        }
+
+        ret = ldb_msg_add_string(msg, SYSDB_SUBDOMAIN_MPG,
+                                 mpg ? "TRUE" : "FALSE");
+        if (ret != LDB_SUCCESS) {
+            ret = sysdb_error_to_errno(ret);
+            goto done;
+        }
+    }
+
+    if (enum_flags) {
+        ret = ldb_msg_add_empty(msg, SYSDB_SUBDOMAIN_ENUM, enum_flags, NULL);
+        if (ret != LDB_SUCCESS) {
+            ret = sysdb_error_to_errno(ret);
+            goto done;
+        }
+
+        ret = ldb_msg_add_string(msg, SYSDB_SUBDOMAIN_ENUM,
+                                 enumerate ? "TRUE" : "FALSE");
+        if (ret != LDB_SUCCESS) {
+            ret = sysdb_error_to_errno(ret);
+            goto done;
+        }
+    }
+
     ret = ldb_modify(sysdb->ldb, msg);
     if (ret != LDB_SUCCESS) {
         DEBUG(SSSDBG_FATAL_FAILURE, ("Failed to add subdomain attributes to "
