@@ -1347,6 +1347,7 @@ static void monitor_hup(struct tevent_context *ev,
         service_signal_rotate(cur_svc);
         if (!strcmp(NSS_SBUS_SERVICE_NAME, cur_svc->name)) {
             service_signal_clear_memcache(cur_svc);
+            service_signal_clear_enum_cache(cur_svc);
         }
 
         if (!strcmp(SSS_AUTOFS_SBUS_SERVICE_NAME, cur_svc->name)) {
@@ -1419,9 +1420,9 @@ static void monitor_quit(struct mt_ctx *mt_ctx, int ret)
                     }
                 } else if (pid != 0) {
                     error = 0;
-                    if WIFEXITED(status) {
+                    if (WIFEXITED(status)) {
                         DEBUG(1, ("Child [%s] exited gracefully\n", svc->name));
-                    } else if WIFSIGNALED(status) {
+                    } else if (WIFSIGNALED(status)) {
                         DEBUG(1, ("Child [%s] terminated with a signal\n", svc->name));
                     } else {
                         DEBUG(0, ("Child [%s] did not exit cleanly\n", svc->name));
@@ -1635,6 +1636,12 @@ done:
     return ret;
 }
 
+errno_t monitor_config_file_fallback(TALLOC_CTX *mem_ctx,
+                                     struct mt_ctx *ctx,
+                                     const char *file,
+                                     monitor_reconf_fn fn,
+                                     bool ignore_missing);
+
 #ifdef HAVE_SYS_INOTIFY_H
 static void process_config_file(struct tevent_context *ev,
                                 struct tevent_timer *te,
@@ -1790,11 +1797,6 @@ done:
     talloc_free(tmp_ctx);
 }
 
-errno_t monitor_config_file_fallback(TALLOC_CTX *mem_ctx,
-                                     struct mt_ctx *ctx,
-                                     const char *file,
-                                     monitor_reconf_fn fn,
-                                     bool ignore_missing);
 static void rewatch_config_file(struct tevent_context *ev,
                                 struct tevent_timer *te,
                                 struct timeval t, void *ptr)
@@ -2572,11 +2574,11 @@ static void mt_svc_exit_handler(int pid, int wait_status, void *pvt)
     struct timeval tv;
     int restart_delay;
 
-    if WIFEXITED(wait_status) {
+    if (WIFEXITED(wait_status)) {
         DEBUG(SSSDBG_OP_FAILURE,
               ("Child [%s] exited with code [%d]\n",
                svc->name, WEXITSTATUS(wait_status)));
-    } else if WIFSIGNALED(wait_status) {
+    } else if (WIFSIGNALED(wait_status)) {
         DEBUG(SSSDBG_OP_FAILURE,
               ("Child [%s] terminated with signal [%d]\n",
                svc->name, WTERMSIG(wait_status)));
