@@ -26,6 +26,8 @@
 #include <sys/stat.h>
 #include <fcntl.h>
 
+#include <sasl/sasl.h>
+
 #include "util/util.h"
 #include "providers/ad/ad_common.h"
 #include "providers/ad/ad_access.h"
@@ -65,12 +67,54 @@ struct bet_ops ad_access_ops = {
     .finalize = NULL
 };
 
+#define AD_COMPAT_ON "1"
+static int ad_sasl_getopt(void *context, const char *plugin_name,
+                          const char *option,
+                          const char **result, unsigned *len)
+{
+    if (!plugin_name || !result) {
+        return SASL_FAIL;
+    }
+    if (strcmp(plugin_name, "GSSAPI") != 0) {
+        return SASL_FAIL;
+    }
+    if (strcmp(option, "ad_compat") != 0) {
+        return SASL_FAIL;
+    }
+    *result = AD_COMPAT_ON;
+    if (len) {
+        *len = 2;
+    }
+    return SASL_OK;
+}
+
+typedef int (*sss_sasl_gen_cb_fn)(void);
+
+static const sasl_callback_t ad_sasl_callbacks[] = {
+    { SASL_CB_GETOPT, (sss_sasl_gen_cb_fn)ad_sasl_getopt, NULL },
+    { SASL_CB_LIST_END, NULL, NULL }
+};
+/* This is quite a hack, we *try* to fool openldap libraries by initializing
+ * sasl first so we can pass in the SASL_CB_GETOPT callback we need to set some
+ * options. Should be removed as soon as openldap exposes a way to do that */
+static void ad_sasl_initialize(void)
+{
+    /* NOTE: this may fail if soe other library in the system happens to
+     * initialize and use openldap libraries or directly the cyrus-sasl
+     * library as this initialization function can be called only once per
+     * process */
+    (void)sasl_client_init(ad_sasl_callbacks);
+}
+
 static errno_t
 common_ad_init(struct be_ctx *bectx)
 {
     errno_t ret;
     char *ad_servers = NULL;
     char *ad_backup_servers = NULL;
+    char *ad_realm;
+
+    ad_sasl_initialize();
 
     /* Get AD-specific options */
     ret = ad_get_common_options(bectx, bectx->cdb,
@@ -86,9 +130,12 @@ common_ad_init(struct be_ctx *bectx)
 
     ad_servers = dp_opt_get_string(ad_options->basic, AD_SERVER);
     ad_backup_servers = dp_opt_get_string(ad_options->basic, AD_BACKUP_SERVER);
+    ad_realm = dp_opt_get_string(ad_options->basic, AD_KRB5_REALM);
 
     /* Set up the failover service */
-    ret = ad_failover_init(ad_options, bectx, ad_servers, ad_backup_servers, ad_options,
+    ret = ad_failover_init(ad_options, bectx, ad_servers, ad_backup_servers, ad_realm,
+                           AD_SERVICE_NAME, AD_GC_SERVICE_NAME,
+                           dp_opt_get_string(ad_options->basic, AD_DOMAIN),
                            &ad_options->service);
     if (ret != EOK) {
         DEBUG(SSSDBG_FATAL_FAILURE,
@@ -109,7 +156,6 @@ sssm_ad_id_init(struct be_ctx *bectx,
 {
     errno_t ret;
     struct ad_id_ctx *ad_ctx;
-    struct sdap_id_ctx *sdap_ctx;
     const char *hostname;
     const char *ad_domain;
     struct ad_srv_plugin_ctx *srv_ctx;
@@ -128,26 +174,14 @@ sssm_ad_id_init(struct be_ctx *bectx,
         return EOK;
     }
 
-    ad_ctx = talloc_zero(ad_options, struct ad_id_ctx);
+
+    ad_ctx = ad_id_ctx_init(ad_options, bectx);
     if (ad_ctx == NULL) {
         return ENOMEM;
     }
-    ad_ctx->ad_options = ad_options;
     ad_options->id_ctx = ad_ctx;
 
-    sdap_ctx = sdap_id_ctx_new(ad_options, bectx, ad_options->service->sdap);
-    if (sdap_ctx == NULL) {
-        return ENOMEM;
-    }
-    ad_ctx->sdap_id_ctx = sdap_ctx;
-    ad_ctx->ldap_ctx = sdap_ctx->conn;
-
-    ad_ctx->gc_ctx = sdap_id_ctx_conn_add(sdap_ctx, ad_options->service->gc);
-    if (ad_ctx->gc_ctx == NULL) {
-        return ENOMEM;
-    }
-
-    ret = ad_dyndns_init(sdap_ctx->be, ad_options);
+    ret = ad_dyndns_init(ad_ctx->sdap_id_ctx->be, ad_options);
     if (ret != EOK) {
         DEBUG(SSSDBG_MINOR_FAILURE,
              ("Failure setting up automatic DNS update\n"));
@@ -165,22 +199,23 @@ sssm_ad_id_init(struct be_ctx *bectx,
     /* Set up various SDAP options */
     ret = ad_get_id_options(ad_options, bectx->cdb,
                             bectx->conf_path,
-                            &sdap_ctx->opts);
+                            &ad_ctx->sdap_id_ctx->opts);
     if (ret != EOK) {
         goto done;
     }
 
-    ret = sdap_id_setup_tasks(sdap_ctx);
+    ret = ldap_id_setup_tasks(ad_ctx->sdap_id_ctx);
     if (ret != EOK) {
         goto done;
     }
 
     /* Set up the ID mapping object */
-    ret = sdap_idmap_init(sdap_ctx, sdap_ctx, &sdap_ctx->opts->idmap_ctx);
+    ret = sdap_idmap_init(ad_ctx->sdap_id_ctx, ad_ctx->sdap_id_ctx,
+                          &ad_ctx->sdap_id_ctx->opts->idmap_ctx);
     if (ret != EOK) goto done;
 
 
-    ret = setup_tls_config(sdap_ctx->opts->basic);
+    ret = setup_tls_config(ad_ctx->sdap_id_ctx->opts->basic);
     if (ret != EOK) {
         DEBUG(SSSDBG_CRIT_FAILURE,
               ("setup_tls_config failed [%s]\n", strerror(ret)));
@@ -217,7 +252,7 @@ sssm_ad_id_init(struct be_ctx *bectx,
     ret = be_refresh_add_cb(bectx->refresh_ctx, BE_REFRESH_TYPE_NETGROUPS,
                             sdap_refresh_netgroups_send,
                             sdap_refresh_netgroups_recv,
-                            sdap_ctx);
+                            ad_ctx->sdap_id_ctx);
     if (ret != EOK && ret != EEXIST) {
         DEBUG(SSSDBG_MINOR_FAILURE, ("Periodical refresh of netgroups "
               "will not work [%d]: %s\n", ret, strerror(ret)));
@@ -262,6 +297,7 @@ sssm_ad_auth_init(struct be_ctx *bectx,
         goto done;
     }
 
+    krb5_auth_ctx->config_type = K5C_GENERIC;
     krb5_auth_ctx->service = ad_options->service->krb5_service;
 
     ret = ad_get_auth_options(krb5_auth_ctx, ad_options, bectx,
