@@ -89,71 +89,7 @@ static errno_t
 ads_store_sdap_subdom(struct ad_subdomains_ctx *ctx,
                       struct sss_domain_info *parent)
 {
-    struct sss_domain_info *dom;
-    struct sdap_domain *sdom, *sditer;
-    char *basedn;
-    errno_t ret;
-
-    for (dom = get_next_domain(parent, true);
-         dom && IS_SUBDOMAIN(dom); /* if we get back to a parent, stop */
-         dom = get_next_domain(dom, false)) {
-
-        DLIST_FOR_EACH(sditer, ctx->sdom) {
-            if (sditer->dom == dom) {
-                break;
-            }
-        }
-
-        if (sditer == NULL) {
-            /* New sdap domain */
-            DEBUG(SSSDBG_TRACE_FUNC, ("subdomain %s is a new one, will "
-                  "create a new sdap domain object\n", dom->name));
-
-            ret = sdap_domain_add(ctx->sdap_id_ctx->opts, dom, &sdom);
-            if (ret != EOK) {
-                DEBUG(SSSDBG_OP_FAILURE,
-                    ("Cannot add new sdap domain for domain %s [%d]: %s\n",
-                    parent->name, ret, strerror(ret)));
-                return ret;
-            }
-            sditer = sdom;
-        }
-
-        /* Convert the domain name into search base */
-        ret = domain_to_basedn(sditer, sditer->dom->name, &basedn);
-        if (ret != EOK) {
-            DEBUG(SSSDBG_OP_FAILURE,
-                ("Cannot convert domain name [%s] to base DN [%d]: %s\n",
-                dom->name, ret, strerror(ret)));
-            talloc_free(basedn);
-            return ret;
-        }
-
-        /* Update search bases */
-        talloc_zfree(sdom->search_bases);
-        sdom->search_bases = talloc_array(sdom, struct sdap_search_base *, 2);
-        if (sdom->search_bases == NULL) {
-            return ret;
-        }
-        sdom->search_bases[1] = NULL;
-
-        ret = sdap_create_search_base(sdom, basedn, LDAP_SCOPE_SUBTREE, NULL,
-                                      &sdom->search_bases[0]);
-        talloc_free(basedn);
-        if (ret) {
-            DEBUG(SSSDBG_OP_FAILURE, ("Cannot create new sdap search base\n"));
-            return ret;
-        }
-
-        sdom->user_search_bases = sdom->search_bases;
-        sdom->group_search_bases = sdom->search_bases;
-        sdom->netgroup_search_bases = sdom->search_bases;
-        sdom->sudo_search_bases = sdom->search_bases;
-        sdom->service_search_bases = sdom->search_bases;
-        sdom->autofs_search_bases = sdom->search_bases;
-    }
-
-    return EOK;
+    return sdap_domain_subdom_add(ctx->sdap_id_ctx, ctx->sdom, parent);
 }
 
 static errno_t
@@ -220,7 +156,9 @@ ad_subdom_store(struct ad_subdomains_ctx *ctx,
         goto done;
     }
 
-    ret = sysdb_subdomain_store(domain->sysdb, name, realm, flat, sid_str);
+    /* AD subdomains are currently all mpg and do not enumerate */
+    ret = sysdb_subdomain_store(domain->sysdb, name, realm, flat, sid_str,
+                                true, false);
     if (ret != EOK) {
         DEBUG(SSSDBG_OP_FAILURE, ("sysdb_subdomain_store failed.\n"));
         goto done;
@@ -631,7 +569,7 @@ static void ad_subdomains_get_netlogon_done(struct tevent_req *req)
     DEBUG(SSSDBG_TRACE_FUNC, ("Found flat name [%s].\n", ctx->flat_name));
 
     ret = sysdb_master_domain_add_info(ctx->sd_ctx->be_ctx->domain,
-                                       NULL, ctx->flat_name, ctx->master_sid);
+                                       ctx->flat_name, ctx->master_sid);
 
     ret = ad_subdomains_get_slave(ctx);
     if (ret == EAGAIN) {
