@@ -38,7 +38,7 @@ struct krb5_access_state {
     bool access_allowed;
 };
 
-static void krb5_access_child_done(struct tevent_req *subreq);
+static void krb5_access_done(struct tevent_req *subreq);
 struct tevent_req *krb5_access_send(TALLOC_CTX *mem_ctx,
                                     struct tevent_context *ev,
                                     struct be_ctx *be_ctx,
@@ -88,8 +88,8 @@ struct tevent_req *krb5_access_send(TALLOC_CTX *mem_ctx,
     attrs[2] = SYSDB_GIDNUM;
     attrs[3] = NULL;
 
-    ret = sysdb_get_user_attr(state, be_ctx->sysdb, state->pd->user, attrs,
-                              &res);
+    ret = sysdb_get_user_attr(state, be_ctx->domain->sysdb, be_ctx->domain,
+                              state->pd->user, attrs, &res);
     if (ret) {
         DEBUG(5, ("sysdb search for upn of user [%s] failed.\n", pd->user));
         goto done;
@@ -103,7 +103,7 @@ struct tevent_req *krb5_access_send(TALLOC_CTX *mem_ctx,
         break;
     case 1:
         ret = find_or_guess_upn(state, res->msgs[0], krb5_ctx,
-                                be_ctx->domain->name, pd->user, pd->domain,
+                                be_ctx->domain, pd->user, pd->domain,
                                 &state->kr->upn);
         if (ret != EOK) {
             DEBUG(SSSDBG_OP_FAILURE, ("find_or_guess_upn failed.\n"));
@@ -141,7 +141,7 @@ struct tevent_req *krb5_access_send(TALLOC_CTX *mem_ctx,
         goto done;
     }
 
-    tevent_req_set_callback(subreq, krb5_access_child_done, req);
+    tevent_req_set_callback(subreq, krb5_access_done, req);
     return req;
 
 done:
@@ -154,7 +154,7 @@ done:
     return req;
 }
 
-static void krb5_access_child_done(struct tevent_req *subreq)
+static void krb5_access_done(struct tevent_req *subreq)
 {
     struct tevent_req *req = tevent_req_callback_data(subreq, struct tevent_req);
     struct krb5_access_state *state = tevent_req_data(req,
