@@ -315,7 +315,7 @@ static void krb5_auth_cache_creds(struct krb5_ctx *krb5_ctx,
     *dp_err = DP_ERR_OFFLINE;
 }
 
-static errno_t krb5_auth_prepare_ccache_file(struct krb5child_req *kr,
+static errno_t krb5_auth_prepare_ccache_name(struct krb5child_req *kr,
                                              struct be_ctx *be_ctx,
                                              int *pam_status, int *dp_err)
 {
@@ -467,6 +467,7 @@ struct tevent_req *krb5_auth_send(TALLOC_CTX *mem_ctx,
     const char *realm;
     struct tevent_req *req;
     struct tevent_req *subreq;
+    int authtok_type;
     int ret;
 
     req = tevent_req_create(mem_ctx, &state, struct krb5_auth_state);
@@ -491,15 +492,28 @@ struct tevent_req *krb5_auth_send(TALLOC_CTX *mem_ctx,
 
     state->sysdb = state->domain->sysdb;
 
+    authtok_type = sss_authtok_get_type(pd->authtok);
+
     switch (pd->cmd) {
         case SSS_PAM_AUTHENTICATE:
         case SSS_PAM_CHAUTHTOK:
-            if (sss_authtok_get_type(pd->authtok) != SSS_AUTHTOK_TYPE_PASSWORD) {
+            if (authtok_type != SSS_AUTHTOK_TYPE_PASSWORD) {
+                /* handle empty password gracefully */
+                if (authtok_type == SSS_AUTHTOK_TYPE_EMPTY) {
+                    DEBUG(SSSDBG_CRIT_FAILURE,
+                          ("Illegal zero-length authtok for user [%s]\n",
+                           pd->user));
+                    state->pam_status = PAM_AUTH_ERR;
+                    state->dp_err = DP_ERR_OK;
+                    ret = EOK;
+                    goto done;
+                }
+
                 DEBUG(SSSDBG_CRIT_FAILURE,
                       ("Wrong authtok type for user [%s]. " \
                        "Expected [%d], got [%d]\n", pd->user,
                           SSS_AUTHTOK_TYPE_PASSWORD,
-                          sss_authtok_get_type(pd->authtok)));
+                          authtok_type));
                 state->pam_status = PAM_SYSTEM_ERR;
                 state->dp_err = DP_ERR_FATAL;
                 ret = EINVAL;
@@ -508,7 +522,7 @@ struct tevent_req *krb5_auth_send(TALLOC_CTX *mem_ctx,
             break;
         case SSS_PAM_CHAUTHTOK_PRELIM:
             if (pd->priv == 1 &&
-                sss_authtok_get_type(pd->authtok) != SSS_AUTHTOK_TYPE_PASSWORD) {
+                authtok_type != SSS_AUTHTOK_TYPE_PASSWORD) {
                 DEBUG(SSSDBG_MINOR_FAILURE,
                       ("Password reset by root is not supported.\n"));
                 state->pam_status = PAM_PERM_DENIED;
@@ -518,12 +532,12 @@ struct tevent_req *krb5_auth_send(TALLOC_CTX *mem_ctx,
             }
             break;
         case SSS_CMD_RENEW:
-            if (sss_authtok_get_type(pd->authtok) != SSS_AUTHTOK_TYPE_CCFILE) {
+            if (authtok_type != SSS_AUTHTOK_TYPE_CCFILE) {
                 DEBUG(SSSDBG_CRIT_FAILURE,
                       ("Wrong authtok type for user [%s]. " \
                        "Expected [%d], got [%d]\n", pd->user,
                           SSS_AUTHTOK_TYPE_CCFILE,
-                          sss_authtok_get_type(pd->authtok)));
+                          authtok_type));
                 state->pam_status = PAM_SYSTEM_ERR;
                 state->dp_err = DP_ERR_FATAL;
                 ret = EINVAL;
@@ -769,7 +783,7 @@ static void krb5_auth_resolve_done(struct tevent_req *subreq)
         }
     }
 
-    ret = krb5_auth_prepare_ccache_file(kr, state->be_ctx,
+    ret = krb5_auth_prepare_ccache_name(kr, state->be_ctx,
                                         &state->pam_status, &state->dp_err);
     if (ret) {
         goto done;
