@@ -436,7 +436,7 @@ done:
     return kerr;
 }
 
-#ifdef HAVE_KRB5_DIRCACHE
+#ifdef HAVE_KRB5_CC_COLLECTION
 static bool need_switch_to_principal(krb5_context ctx, krb5_principal princ)
 {
     krb5_error_code kerr;
@@ -447,7 +447,7 @@ static bool need_switch_to_principal(krb5_context ctx, krb5_principal princ)
     bool ret = false;
 
     kerr = krb5_cc_default(ctx, &default_cc);
-    if (kerr !=0) {
+    if (kerr != 0) {
         KRB5_CHILD_DEBUG(SSSDBG_TRACE_INTERNAL, kerr);
         goto done;
     }
@@ -463,13 +463,13 @@ static bool need_switch_to_principal(krb5_context ctx, krb5_principal princ)
     }
 
     kerr = krb5_unparse_name(ctx, default_princ, &default_full_name);
-    if (kerr !=0) {
+    if (kerr != 0) {
         KRB5_CHILD_DEBUG(SSSDBG_TRACE_INTERNAL, kerr);
         goto done;
     }
 
     kerr = krb5_unparse_name(ctx, princ, &full_name);
-    if (kerr !=0) {
+    if (kerr != 0) {
         KRB5_CHILD_DEBUG(SSSDBG_TRACE_INTERNAL, kerr);
         goto done;
     }
@@ -497,7 +497,7 @@ done:
 
     return ret;
 }
-#endif /* HAVE_KRB5_DIRCACHE */
+#endif /* HAVE_KRB5_CC_COLLECTION */
 
 static krb5_error_code
 store_creds_in_ccache(krb5_context ctx, krb5_principal princ,
@@ -505,6 +505,19 @@ store_creds_in_ccache(krb5_context ctx, krb5_principal princ,
 {
     krb5_error_code kerr;
     krb5_creds *l_cred;
+    char *ccname;
+
+    if (DEBUG_IS_SET(SSSDBG_TRACE_ALL)) {
+        kerr = krb5_cc_get_full_name(ctx, cc, &ccname);
+        if (kerr != 0) {
+            DEBUG(SSSDBG_TRACE_ALL,
+                  ("Couldn't determine full name of ccache\n"));
+        } else {
+            DEBUG(SSSDBG_TRACE_ALL,
+                  ("Storing credentials in [%s]\n", ccname));
+            krb5_free_string(ctx, ccname);
+        }
+    }
 
     kerr = krb5_cc_initialize(ctx, cc, princ);
     if (kerr != 0) {
@@ -528,7 +541,7 @@ store_creds_in_ccache(krb5_context ctx, krb5_principal princ,
         goto done;
     }
 
-#ifdef HAVE_KRB5_DIRCACHE
+#ifdef HAVE_KRB5_CC_COLLECTION
     if (need_switch_to_principal(ctx, princ)) {
         kerr = krb5_cc_switch(ctx, cc);
         if (kerr != 0) {
@@ -536,7 +549,7 @@ store_creds_in_ccache(krb5_context ctx, krb5_principal princ,
             goto done;
         }
     }
-#endif /* HAVE_KRB5_DIRCACHE */
+#endif /* HAVE_KRB5_CC_COLLECTION */
 
     kerr = krb5_cc_close(ctx, cc);
     if (kerr != 0) {
@@ -654,7 +667,7 @@ done:
     return kerr;
 }
 
-#ifdef HAVE_KRB5_DIRCACHE
+#ifdef HAVE_KRB5_CC_COLLECTION
 
 static errno_t
 create_ccdir(const char *dirname, uid_t uid, gid_t gid)
@@ -722,12 +735,6 @@ create_ccache_in_dir(uid_t uid, gid_t gid,
         return EIO;
     }
 
-    kerr = become_user(uid, gid);
-    if (kerr != EOK) {
-        DEBUG(SSSDBG_CRIT_FAILURE, ("become_user failed.\n"));
-        goto done;
-    }
-
     if (dirname[0] == ':') {
         /* Cache name in the form of DIR::filepath represents a single
          * ccache in a collection that we are trying to reuse.
@@ -781,7 +788,38 @@ done:
     return kerr;
 }
 
-#endif /* HAVE_KRB5_DIRCACHE */
+static krb5_error_code
+create_ccache_keyring(krb5_context ctx,
+                      krb5_principal princ,
+                      char *ccname,
+                      krb5_creds *creds)
+{
+    krb5_error_code kerr;
+    krb5_ccache tmp_cc = NULL;
+
+    DEBUG(SSSDBG_FUNC_DATA, ("Creating ccache at [%s]\n", ccname));
+
+    kerr = krb5_cc_resolve(ctx, ccname, &tmp_cc);
+    if (kerr != 0) {
+        KRB5_CHILD_DEBUG(SSSDBG_CRIT_FAILURE, kerr);
+        goto done;
+    }
+
+    kerr = store_creds_in_ccache(ctx, princ, tmp_cc, creds);
+    if (kerr != 0) {
+        KRB5_CHILD_DEBUG(SSSDBG_CRIT_FAILURE, kerr);
+        goto done;
+    }
+
+done:
+    if (kerr != 0 && tmp_cc != NULL) {
+        krb5_cc_destroy(ctx, tmp_cc);
+    }
+
+    return kerr;
+}
+
+#endif /* HAVE_KRB5_CC_COLLECTION */
 
 static krb5_error_code
 create_ccache(uid_t uid, gid_t gid, krb5_context ctx,
@@ -793,10 +831,15 @@ create_ccache(uid_t uid, gid_t gid, krb5_context ctx,
     switch (cctype) {
         case SSS_KRB5_TYPE_FILE:
             return create_ccache_file(ctx, princ, ccname, creds);
-#ifdef HAVE_KRB5_DIRCACHE
+
+#ifdef HAVE_KRB5_CC_COLLECTION
         case SSS_KRB5_TYPE_DIR:
             return create_ccache_in_dir(uid, gid, ctx, princ, ccname, creds);
-#endif /* HAVE_KRB5_DIRCACHE */
+
+        case SSS_KRB5_TYPE_KEYRING:
+            return create_ccache_keyring(ctx, princ, ccname, creds);
+#endif /* HAVE_KRB5_CC_COLLECTION */
+
         default:
             DEBUG(SSSDBG_CRIT_FAILURE, ("Unknown cache type\n"));
             return EINVAL;
@@ -1147,51 +1190,6 @@ done:
 
 }
 
-static char * get_ccache_name_by_principal(TALLOC_CTX *mem_ctx,
-                                           krb5_context ctx,
-                                           krb5_principal principal,
-                                           const char *ccname)
-{
-    krb5_error_code kerr;
-    krb5_ccache tmp_cc = NULL;
-    char *tmp_ccname = NULL;
-    char *ret_ccname = NULL;
-
-    kerr = krb5_cc_set_default_name(ctx, ccname);
-    if (kerr != 0) {
-        KRB5_CHILD_DEBUG(SSSDBG_MINOR_FAILURE, kerr);
-        return NULL;
-    }
-
-    kerr = krb5_cc_cache_match(ctx, principal, &tmp_cc);
-    if (kerr != 0) {
-        KRB5_CHILD_DEBUG(SSSDBG_TRACE_INTERNAL, kerr);
-        return NULL;
-    }
-
-    kerr = krb5_cc_get_full_name(ctx, tmp_cc, &tmp_ccname);
-    if (kerr !=0) {
-        KRB5_CHILD_DEBUG(SSSDBG_MINOR_FAILURE, kerr);
-        goto done;
-    }
-
-    ret_ccname = talloc_strdup(mem_ctx, tmp_ccname);
-    if (ret_ccname == NULL) {
-        DEBUG(SSSDBG_OP_FAILURE, ("talloc_strdup failed (ENOMEM).\n"));
-    }
-
-done:
-    if (tmp_cc != NULL) {
-        kerr = krb5_cc_close(ctx, tmp_cc);
-        if (kerr != 0) {
-            KRB5_CHILD_DEBUG(SSSDBG_MINOR_FAILURE, kerr);
-        }
-    }
-    krb5_free_string(ctx, tmp_ccname);
-
-    return ret_ccname;
-}
-
 static krb5_error_code get_and_save_tgt(struct krb5_req *kr,
                                         const char *password)
 {
@@ -1250,8 +1248,8 @@ static krb5_error_code get_and_save_tgt(struct krb5_req *kr,
      * directly with file ccache (DIR::/...), but cache collection
      * should be returned back to back end.
      */
-    cc_name = get_ccache_name_by_principal(kr->pd, kr->ctx, principal,
-                                           kr->ccname);
+    cc_name = sss_get_ccache_name_for_principal(kr->pd, kr->ctx, principal,
+                                                kr->ccname);
     if (cc_name == NULL) {
         cc_name = kr->ccname;
     }
@@ -1280,7 +1278,9 @@ done:
 
 static errno_t map_krb5_error(krb5_error_code kerr)
 {
-    KRB5_CHILD_DEBUG(SSSDBG_CRIT_FAILURE, kerr);
+    if (kerr != 0) {
+        KRB5_CHILD_DEBUG(SSSDBG_CRIT_FAILURE, kerr);
+    }
 
     switch (kerr) {
     case 0:
@@ -1323,6 +1323,8 @@ static errno_t changepw_child(struct krb5_req *kr, bool prelim)
     const char *realm_name;
     int realm_length;
     krb5_get_init_creds_opt *chagepw_options;
+    size_t msg_len;
+    uint8_t *msg;
 
     DEBUG(SSSDBG_TRACE_LIBS, ("Password change operation\n"));
 
@@ -1355,6 +1357,19 @@ static errno_t changepw_child(struct krb5_req *kr, bool prelim)
                                         chagepw_options);
     sss_krb5_get_init_creds_opt_free(kr->ctx, chagepw_options);
     if (kerr != 0) {
+        ret = pack_user_info_chpass_error(kr->pd, "Old password not accepted.",
+                                          &msg_len, &msg);
+        if (ret != EOK) {
+            DEBUG(SSSDBG_CRIT_FAILURE,
+                  ("pack_user_info_chpass_error failed.\n"));
+        } else {
+            ret = pam_add_response(kr->pd, SSS_PAM_USER_INFO, msg_len,
+                                   msg);
+            if (ret != EOK) {
+                DEBUG(SSSDBG_CRIT_FAILURE,
+                      ("pam_add_response failed.\n"));
+            }
+        }
         return kerr;
     }
 
@@ -1400,7 +1415,7 @@ static errno_t changepw_child(struct krb5_req *kr, bool prelim)
             }
         }
 
-        if (result_string.length > 0) {
+        if (result_string.length > 0 && result_string.data[0] != '\0') {
             DEBUG(1, ("krb5_change_password failed [%d][%.*s].\n", result_code,
                       result_string.length, result_string.data));
             talloc_free(user_error_message);
@@ -1409,6 +1424,12 @@ static errno_t changepw_child(struct krb5_req *kr, bool prelim)
             if (user_error_message == NULL) {
                 DEBUG(1, ("talloc_strndup failed.\n"));
             }
+        } else if (result_code == KRB5_KPASSWD_SOFTERROR) {
+            user_error_message = talloc_strdup(kr->pd, "Please make sure the "
+                                 "password meets the complexity constraints.");
+            if (user_error_message == NULL) {
+                DEBUG(SSSDBG_CRIT_FAILURE, ("talloc_strndup failed.\n"));
+            }
         }
 
         if (user_error_message != NULL) {
@@ -1451,12 +1472,18 @@ static errno_t tgt_req_child(struct krb5_req *kr)
 
     ret = sss_authtok_get_password(kr->pd->authtok, &password, NULL);
     switch (ret) {
-        if (ret == EACCES) {
+        case EOK:
+            break;
+
+        case EACCES:
             DEBUG(SSSDBG_OP_FAILURE, ("Invalid authtok type\n"));
             return ERR_INVALID_CRED_TYPE;
-        }
-        DEBUG(SSSDBG_OP_FAILURE, ("No credentials available\n"));
-        return ERR_NO_CREDS;
+            break;
+
+        default:
+            DEBUG(SSSDBG_OP_FAILURE, ("No credentials available\n"));
+            return ERR_NO_CREDS;
+            break;
     }
 
     kerr = get_and_save_tgt(kr, password);
@@ -1635,7 +1662,7 @@ static errno_t create_empty_ccache(struct krb5_req *kr)
     return map_krb5_error(kerr);
 }
 
-static errno_t unpack_authtok(TALLOC_CTX *mem_ctx, struct sss_auth_token *tok,
+static errno_t unpack_authtok(struct sss_auth_token *tok,
                               uint8_t *buf, size_t size, size_t *p)
 {
     uint32_t auth_token_type;
@@ -1728,7 +1755,7 @@ static errno_t unpack_buffer(uint8_t *buf, size_t size,
         if (kr->keytab == NULL) return ENOMEM;
         p += len;
 
-        ret = unpack_authtok(pd, pd->authtok, buf, size, &p);
+        ret = unpack_authtok(pd->authtok, buf, size, &p);
         if (ret) {
             return ret;
         }
@@ -1742,7 +1769,7 @@ static errno_t unpack_buffer(uint8_t *buf, size_t size,
     }
 
     if (pd->cmd == SSS_PAM_CHAUTHTOK) {
-        ret = unpack_authtok(pd, pd->newauthtok, buf, size, &p);
+        ret = unpack_authtok(pd->newauthtok, buf, size, &p);
         if (ret) {
             return ret;
         }
@@ -1853,8 +1880,7 @@ static krb5_error_code check_fast_ccache(TALLOC_CTX *mem_ctx,
     tmp_ctx = talloc_new(NULL);
     if (tmp_ctx == NULL) {
         DEBUG(1, ("talloc_new failed.\n"));
-        kerr = ENOMEM;
-        goto done;
+        return ENOMEM;
     }
 
     ccname = talloc_asprintf(tmp_ctx, "FILE:%s/fast_ccache_%s", DB_PATH, realm);
@@ -2061,7 +2087,7 @@ static int k5c_setup(struct krb5_req *kr, uint32_t offline)
 
     if (debug_level & SSSDBG_TRACE_ALL) {
         kerr = sss_child_set_krb5_tracing(kr->ctx);
-        if (kerr) {
+        if (kerr != 0) {
             KRB5_CHILD_DEBUG(SSSDBG_MINOR_FAILURE, kerr);
             return EIO;
         }
