@@ -142,11 +142,17 @@ static errno_t create_send_buffer(struct krb5child_req *kr,
 
     validate = dp_opt_get_bool(kr->krb5_ctx->opts, KRB5_VALIDATE) ? 1 : 0;
 
-    /* Always send PAC except for local IPA users */
-    if (kr->krb5_ctx->is_ipa) {
-        send_pac = kr->upn_from_different_realm ? 1 : 0;
-    } else {
-        send_pac = 1;
+    /* Always send PAC except for local IPA users and IPA server mode */
+    switch (kr->krb5_ctx->config_type) {
+        case K5C_IPA_CLIENT:
+            send_pac = kr->upn_from_different_realm ? 1 : 0;
+            break;
+        case K5C_IPA_SERVER:
+            send_pac = 0;
+            break;
+        default:
+            send_pac = 1;
+            break;
     }
 
     if (kr->pd->cmd == SSS_CMD_RENEW) {
@@ -329,7 +335,7 @@ static errno_t fork_child(struct tevent_req *req)
         fd_nonblocking(state->io->read_from_child_fd);
         fd_nonblocking(state->io->write_to_child_fd);
 
-        ret = child_handler_setup(state->ev, pid, NULL, NULL);
+        ret = child_handler_setup(state->ev, pid, NULL, NULL, NULL);
         if (ret != EOK) {
             DEBUG(1, ("Could not set up child signal handler\n"));
             return ret;
@@ -486,7 +492,7 @@ parse_krb5_child_response(TALLOC_CTX *mem_ctx, uint8_t *buf, ssize_t len,
     errno_t ret;
     bool skip;
     char *ccname = NULL;
-    size_t ccname_len;
+    size_t ccname_len = 0;
     int32_t msg_status;
     int32_t msg_type;
     int32_t msg_len;
@@ -496,7 +502,7 @@ parse_krb5_child_response(TALLOC_CTX *mem_ctx, uint8_t *buf, ssize_t len,
     uint32_t *msg_subtype;
     struct krb5_child_response *res;
     const char *upn = NULL;
-    size_t upn_len;
+    size_t upn_len = 0;
     bool otp = false;
 
     if ((size_t) len < sizeof(int32_t)) {
