@@ -33,209 +33,134 @@
 #include "providers/ldap/sdap_async.h"
 
 /* ==Cleanup-Task========================================================= */
+struct ldap_id_cleanup_ctx {
+    struct sdap_id_ctx *ctx;
+    struct sdap_domain *sdom;
+};
 
-struct tevent_req *ldap_id_cleanup_send(TALLOC_CTX *memctx,
-                                        struct tevent_context *ev,
-                                        struct sdap_id_ctx *ctx);
-static void ldap_id_cleanup_reschedule(struct tevent_req *req);
-
-static void ldap_id_cleanup_timeout(struct tevent_context *ev,
-                                      struct tevent_timer *te,
-                                      struct timeval tv, void *pvt);
+static errno_t ldap_id_cleanup_set_timer(struct ldap_id_cleanup_ctx *cctx,
+                                         struct timeval tv);
 
 static void ldap_id_cleanup_timer(struct tevent_context *ev,
                                   struct tevent_timer *tt,
                                   struct timeval tv, void *pvt)
 {
-    struct sdap_id_ctx *ctx = talloc_get_type(pvt, struct sdap_id_ctx);
-    struct tevent_timer *timeout;
-    struct tevent_req *req;
+    struct ldap_id_cleanup_ctx *cctx = talloc_get_type(pvt,
+                                                struct ldap_id_cleanup_ctx);
     int delay;
     errno_t ret;
 
-    if (be_is_offline(ctx->be)) {
-        DEBUG(4, ("Backend is marked offline, retry later!\n"));
+    if (be_is_offline(cctx->ctx->be)) {
+        DEBUG(SSSDBG_TRACE_FUNC, ("Backend is marked offline, retry later!\n"));
         /* schedule starting from now, not the last run */
-        delay = dp_opt_get_int(ctx->opts->basic, SDAP_CACHE_PURGE_TIMEOUT);
+        delay = dp_opt_get_int(cctx->ctx->opts->basic,
+                               SDAP_CACHE_PURGE_TIMEOUT);
         tv = tevent_timeval_current_ofs(delay, 0);
-        ldap_id_cleanup_set_timer(ctx, tv);
+        ldap_id_cleanup_set_timer(cctx, tv);
         return;
     }
 
-    req = ldap_id_cleanup_send(ctx, ev, ctx);
-    if (!req) {
-        DEBUG(1, ("Failed to schedule cleanup, retrying later!\n"));
-        /* schedule starting from now, not the last run */
-        delay = dp_opt_get_int(ctx->opts->basic, SDAP_CACHE_PURGE_TIMEOUT);
-        tv = tevent_timeval_current_ofs(delay, 0);
-        ret = ldap_id_cleanup_set_timer(ctx, tv);
-        if (ret != EOK) {
-            DEBUG(1, ("Error setting up cleanup timer\n"));
-        }
-        return;
-    }
-    tevent_req_set_callback(req, ldap_id_cleanup_reschedule, ctx);
-
-    /* if cleanup takes so long, either we try to cleanup too
-     * frequently, or something went seriously wrong */
-    delay = dp_opt_get_int(ctx->opts->basic, SDAP_CACHE_PURGE_TIMEOUT);
-    tv = tevent_timeval_current_ofs(delay, 0);
-    timeout = tevent_add_timer(ctx->be->ev, req, tv,
-                               ldap_id_cleanup_timeout, req);
-    if (timeout == NULL) {
-        /* If we can't guarantee a timeout, we
-         * need to cancel the request, to avoid
-         * the possibility of starting another
-         * concurrently
-         */
-        talloc_zfree(req);
-
-        DEBUG(1, ("Failed to schedule cleanup, retrying later!\n"));
-        /* schedule starting from now, not the last run */
-        delay = dp_opt_get_int(ctx->opts->basic, SDAP_CACHE_PURGE_TIMEOUT);
-        tv = tevent_timeval_current_ofs(delay, 0);
-        ret = ldap_id_cleanup_set_timer(ctx, tv);
-        if (ret != EOK) {
-            DEBUG(1, ("Error setting up cleanup timer\n"));
-        }
-        return;
-    }
-    return;
-}
-
-static void ldap_id_cleanup_timeout(struct tevent_context *ev,
-                                      struct tevent_timer *te,
-                                      struct timeval tv, void *pvt)
-{
-    struct tevent_req *req = talloc_get_type(pvt, struct tevent_req);
-    struct sdap_id_ctx *ctx = tevent_req_callback_data(req,
-                                                       struct sdap_id_ctx);
-    int delay;
-
-    delay = dp_opt_get_int(ctx->opts->basic, SDAP_CACHE_PURGE_TIMEOUT);
-    DEBUG(1, ("Cleanup timed out! Timeout too small? (%ds)!\n", delay));
-
-    tv = tevent_timeval_current_ofs(delay, 0);
-    ldap_id_cleanup_set_timer(ctx, tv);
-
-    talloc_zfree(req);
-}
-
-static void ldap_id_cleanup_reschedule(struct tevent_req *req)
-{
-    struct sdap_id_ctx *ctx = tevent_req_callback_data(req,
-                                                       struct sdap_id_ctx);
-    enum tevent_req_state tstate;
-    uint64_t err;
-    struct timeval tv;
-    int delay;
-
-    if (tevent_req_is_error(req, &tstate, &err)) {
+    ret = ldap_id_cleanup(cctx->ctx->opts, cctx->sdom->dom);
+    if (ret != EOK) {
         /* On error schedule starting from now, not the last run */
         tv = tevent_timeval_current();
     } else {
-        tv = ctx->last_purge;
+        tv = cctx->sdom->last_purge;
     }
-    talloc_zfree(req);
 
-    delay = dp_opt_get_int(ctx->opts->basic, SDAP_CACHE_PURGE_TIMEOUT);
+    delay = dp_opt_get_int(cctx->ctx->opts->basic, SDAP_CACHE_PURGE_TIMEOUT);
     tv = tevent_timeval_add(&tv, delay, 0);
-    ldap_id_cleanup_set_timer(ctx, tv);
+    ldap_id_cleanup_set_timer(cctx, tv);
+    cctx->sdom->last_purge = tevent_timeval_current();
 }
 
-
-
-int ldap_id_cleanup_set_timer(struct sdap_id_ctx *ctx, struct timeval tv)
+static errno_t ldap_id_cleanup_set_timer(struct ldap_id_cleanup_ctx *cctx,
+                                         struct timeval tv)
 {
     struct tevent_timer *cleanup_task;
 
-    DEBUG(6, ("Scheduling next cleanup at %ld.%ld\n",
-              (long)tv.tv_sec, (long)tv.tv_usec));
-
-    cleanup_task = tevent_add_timer(ctx->be->ev, ctx,
-                                    tv, ldap_id_cleanup_timer, ctx);
-    if (!cleanup_task) {
-        DEBUG(0, ("FATAL: failed to setup cleanup task!\n"));
+    cleanup_task = tevent_add_timer(cctx->ctx->be->ev, cctx->ctx,
+                                    tv, ldap_id_cleanup_timer, cctx);
+    if (cleanup_task == NULL) {
+        DEBUG(SSSDBG_CRIT_FAILURE, ("FATAL: failed to setup cleanup task!\n"));
         return EFAULT;
     }
 
     return EOK;
 }
 
+int ldap_id_cleanup_create_timer(struct sdap_id_ctx *ctx,
+                                 struct sdap_domain *sdom,
+                                 struct timeval tv)
+{
+    struct ldap_id_cleanup_ctx *cctx;
 
+    DEBUG(SSSDBG_FUNC_DATA,
+          ("Scheduling next cleanup at %ld.%ld\n",
+          (long)tv.tv_sec, (long)tv.tv_usec));
 
-struct global_cleanup_state {
-    struct tevent_context *ev;
-    struct sdap_id_ctx *ctx;
-};
+    cctx = talloc(ctx, struct ldap_id_cleanup_ctx);
+    if (cctx == NULL) {
+        return ENOMEM;
+    }
+    cctx->ctx = ctx;
+    cctx->sdom = sdom;
 
-static int cleanup_users(TALLOC_CTX *memctx, struct sdap_id_ctx *ctx);
+    return ldap_id_cleanup_set_timer(cctx, tv);
+}
+
+static int cleanup_users(struct sdap_options *opts,
+                         struct sss_domain_info *dom);
 static int cleanup_groups(TALLOC_CTX *memctx,
                           struct sysdb_ctx *sysdb,
                           struct sss_domain_info *domain);
 
-struct tevent_req *ldap_id_cleanup_send(TALLOC_CTX *memctx,
-                                        struct tevent_context *ev,
-                                        struct sdap_id_ctx *ctx)
+errno_t ldap_id_cleanup(struct sdap_options *opts,
+                        struct sss_domain_info *dom)
 {
-    struct global_cleanup_state *state;
-    struct tevent_req *req;
-    int ret;
+    int ret, tret;
     bool in_transaction = false;
+    TALLOC_CTX *tmp_ctx;
 
-    req = tevent_req_create(memctx, &state, struct global_cleanup_state);
-    if (!req) return NULL;
+    tmp_ctx = talloc_new(NULL);
+    if (tmp_ctx == NULL) {
+        return ENOMEM;
+    }
 
-    state->ev = ev;
-    state->ctx = ctx;
-
-    ctx->last_purge = tevent_timeval_current();
-
-    ret = sysdb_transaction_start(state->ctx->be->domain->sysdb);
+    ret = sysdb_transaction_start(dom->sysdb);
     if (ret != EOK) {
         DEBUG(SSSDBG_CRIT_FAILURE, ("Failed to start transaction\n"));
-        goto fail;
+        goto done;
     }
     in_transaction = true;
 
-    ret = cleanup_users(state, state->ctx);
+    ret = cleanup_users(opts, dom);
     if (ret && ret != ENOENT) {
-        goto fail;
+        goto done;
     }
 
-    ret = cleanup_groups(state,
-                         state->ctx->be->domain->sysdb,
-                         state->ctx->be->domain);
+    ret = cleanup_groups(tmp_ctx, dom->sysdb, dom);
     if (ret) {
-        goto fail;
+        goto done;
     }
 
-    ret = sysdb_transaction_commit(state->ctx->be->domain->sysdb);
+    ret = sysdb_transaction_commit(dom->sysdb);
     if (ret != EOK) {
         DEBUG(SSSDBG_CRIT_FAILURE, ("Failed to commit transaction\n"));
-        goto fail;
+        goto done;
     }
     in_transaction = false;
 
-    tevent_req_done(req);
-    tevent_req_post(req, ev);
-    return req;
-
-fail:
-    DEBUG(1, ("Failed to cleanup caches (%d [%s]), retrying later!\n",
-              (int)ret, strerror(ret)));
+    ret = EOK;
+done:
     if (in_transaction) {
-        ret = sysdb_transaction_cancel(state->ctx->be->domain->sysdb);
-        if (ret != EOK) {
-            DEBUG(1, ("Could not cancel transaction\n"));
-            tevent_req_error(req, ret);
-            tevent_req_post(req, ev);
-            return req;
+        tret = sysdb_transaction_cancel(dom->sysdb);
+        if (tret != EOK) {
+            DEBUG(SSSDBG_CRIT_FAILURE, ("Could not cancel transaction\n"));
         }
     }
-    tevent_req_done(req);
-    tevent_req_post(req, ev);
-    return req;
+    talloc_free(tmp_ctx);
+    return ret;
 }
 
 
@@ -244,10 +169,11 @@ fail:
 static int cleanup_users_logged_in(hash_table_t *table,
                                    const struct ldb_message *msg);
 
-static int cleanup_users(TALLOC_CTX *memctx, struct sdap_id_ctx *ctx)
+static int cleanup_users(struct sdap_options *opts,
+                         struct sss_domain_info *dom)
 {
     TALLOC_CTX *tmpctx;
-    struct sysdb_ctx *sysdb = ctx->be->domain->sysdb;
+    struct sysdb_ctx *sysdb = dom->sysdb;
     const char *attrs[] = { SYSDB_NAME, SYSDB_UIDNUM, NULL };
     time_t now = time(NULL);
     char *subfilter = NULL;
@@ -259,13 +185,12 @@ static int cleanup_users(TALLOC_CTX *memctx, struct sdap_id_ctx *ctx)
     int ret;
     int i;
 
-    tmpctx = talloc_new(memctx);
+    tmpctx = talloc_new(NULL);
     if (!tmpctx) {
         return ENOMEM;
     }
 
-    account_cache_expiration = dp_opt_get_int(ctx->opts->basic,
-                                           SDAP_ACCOUNT_CACHE_EXPIRATION);
+    account_cache_expiration = dp_opt_get_int(opts->basic, SDAP_ACCOUNT_CACHE_EXPIRATION);
     DEBUG(9, ("Cache expiration is set to %d days\n",
               account_cache_expiration));
 
@@ -292,7 +217,7 @@ static int cleanup_users(TALLOC_CTX *memctx, struct sdap_id_ctx *ctx)
         goto done;
     }
 
-    ret = sysdb_search_users(tmpctx, sysdb, ctx->be->domain,
+    ret = sysdb_search_users(tmpctx, sysdb, dom,
                              subfilter, attrs, &count, &msgs);
     if (ret) {
         if (ret == ENOENT) {
@@ -339,7 +264,7 @@ static int cleanup_users(TALLOC_CTX *memctx, struct sdap_id_ctx *ctx)
 
         /* If not logged in or cannot check the table, delete him */
         DEBUG(9, ("About to delete user %s\n", name));
-        ret = sysdb_delete_user(sysdb, ctx->be->domain, name, 0);
+        ret = sysdb_delete_user(sysdb, dom, name, 0);
         if (ret) {
             goto done;
         }
