@@ -219,6 +219,8 @@ enum sdap_basic_opt {
     SDAP_AD_MATCHING_RULE_INITGROUPS,
     SDAP_RFC2307_FALLBACK_TO_LOCAL_USERS,
     SDAP_DISABLE_RANGE_RETRIEVAL,
+    SDAP_MIN_ID,
+    SDAP_MAX_ID,
 
     SDAP_OPTS_BASIC /* opts counter */
 };
@@ -381,6 +383,13 @@ struct sdap_domain {
     struct sdap_domain *next, *prev;
     /* Need to modify the list from a talloc destructor */
     struct sdap_domain **head;
+
+    /* Enumeration and cleanup periodic task */
+    struct be_ptask *enum_task;
+    /* enumeration loop timer */
+    struct timeval last_enum;
+    /* cleanup loop timer */
+    struct timeval last_purge;
 };
 
 struct sdap_options {
@@ -436,6 +445,11 @@ struct sdap_deref_attrs {
     struct sysdb_attrs *attrs;
 };
 
+int sdap_copy_map(TALLOC_CTX *memctx,
+                 struct sdap_attr_map *src_map,
+                 int num_entries,
+                 struct sdap_attr_map **_map);
+
 int sdap_get_map(TALLOC_CTX *memctx,
                  struct confdb_ctx *cdb,
                  const char *conf_path,
@@ -485,6 +499,18 @@ int sdap_control_create(struct sdap_handle *sh, const char *oid, int iscritical,
 
 int sdap_replace_id(struct sysdb_attrs *entry, const char *attr, id_t val);
 
+errno_t sdap_get_group_primary_name(TALLOC_CTX *memctx,
+                                    struct sdap_options *opts,
+                                    struct sysdb_attrs *attrs,
+                                    struct sss_domain_info *dom,
+                                    const char **_group_name);
+
+errno_t sdap_get_user_primary_name(TALLOC_CTX *memctx,
+                                   struct sdap_options *opts,
+                                   struct sysdb_attrs *attrs,
+                                   struct sss_domain_info *dom,
+                                   const char **_user_name);
+
 errno_t sdap_set_config_options_with_rootdse(struct sysdb_attrs *rootdse,
                                              struct sdap_options *opts,
                                              struct sdap_domain *sdom);
