@@ -39,7 +39,7 @@ static errno_t sdap_add_incomplete_groups(struct sysdb_ctx *sysdb,
     TALLOC_CTX *tmp_ctx;
     struct ldb_message *msg;
     int i, mi, ai;
-    const char *name;
+    const char *groupname;
     const char *original_dn;
     char **missing;
     gid_t gid;
@@ -48,8 +48,9 @@ static errno_t sdap_add_incomplete_groups(struct sysdb_ctx *sysdb,
     bool in_transaction = false;
     bool posix;
     time_t now;
-    char *sid_str;
-    bool use_id_mapping = dp_opt_get_bool(opts->basic, SDAP_ID_MAPPING);
+    char *sid_str = NULL;
+    bool use_id_mapping;
+    char *tmp_name;
 
     /* There are no groups in LDAP but we should add user to groups ?? */
     if (ldap_groups_count == 0) return EOK;
@@ -65,14 +66,23 @@ static errno_t sdap_add_incomplete_groups(struct sysdb_ctx *sysdb,
     mi = 0;
 
     for (i=0; groupnames[i]; i++) {
+        tmp_name = sss_get_domain_name(tmp_ctx, groupnames[i], domain);
+        if (tmp_name == NULL) {
+            DEBUG(SSSDBG_OP_FAILURE,
+                  ("Failed to format original name [%s]\n", groupnames[i]));
+            ret = ENOMEM;
+            goto done;
+        }
+
         ret = sysdb_search_group_by_name(tmp_ctx, sysdb, domain,
-                                         groupnames[i], NULL, &msg);
+                                         tmp_name, NULL, &msg);
         if (ret == EOK) {
             continue;
         } else if (ret == ENOENT) {
-            DEBUG(7, ("Group #%d [%s] is not cached, need to add a fake entry\n",
-                       i, groupnames[i]));
-            missing[mi] = groupnames[i];
+            missing[mi] = talloc_steal(missing, tmp_name);
+            DEBUG(7, ("Group #%d [%s][%s] is not cached, " \
+                      "need to add a fake entry\n",
+                      i, groupnames[i], missing[mi]));
             mi++;
             continue;
         } else if (ret != ENOENT) {
@@ -89,6 +99,9 @@ static errno_t sdap_add_incomplete_groups(struct sysdb_ctx *sysdb,
         goto done;
     }
 
+    use_id_mapping = sdap_idmap_domain_has_algorithmic_mapping(opts->idmap_ctx,
+                                                             domain->domain_id);
+
     ret = sysdb_transaction_start(sysdb);
     if (ret != EOK) {
         DEBUG(SSSDBG_CRIT_FAILURE,
@@ -103,30 +116,38 @@ static errno_t sdap_add_incomplete_groups(struct sysdb_ctx *sysdb,
     for (i=0; missing[i]; i++) {
         /* The group is not in sysdb, need to add a fake entry */
         for (ai=0; ai < ldap_groups_count; ai++) {
-            ret = sysdb_attrs_primary_name(sysdb, ldap_groups[ai],
-                                           opts->group_map[SDAP_AT_GROUP_NAME].name,
-                                           &name);
+            ret = sdap_get_group_primary_name(tmp_ctx, opts, ldap_groups[ai],
+                                              domain, &groupname);
             if (ret != EOK) {
-                DEBUG(1, ("The group has no name attribute\n"));
+                DEBUG(SSSDBG_CRIT_FAILURE,
+                      ("The group has no name attribute\n"));
                 goto done;
             }
 
-            if (strcmp(name, missing[i]) == 0) {
+            if (strcmp(groupname, missing[i]) == 0) {
                 posix = true;
 
-                if (use_id_mapping) {
-                    DEBUG(SSSDBG_TRACE_LIBS,
-                          ("Mapping group [%s] objectSID to unix ID\n", name));
+                ret = sdap_attrs_get_sid_str(
+                        tmp_ctx, opts->idmap_ctx, ldap_groups[ai],
+                        opts->group_map[SDAP_AT_GROUP_OBJECTSID].sys_name,
+                        &sid_str);
+                if (ret != EOK && ret != ENOENT) goto done;
 
-                    ret = sdap_attrs_get_sid_str(
-                            tmp_ctx, opts->idmap_ctx, ldap_groups[ai],
-                            opts->group_map[SDAP_AT_GROUP_OBJECTSID].sys_name,
-                            &sid_str);
-                    if (ret != EOK) goto done;
+                if (use_id_mapping) {
+                    if (sid_str == NULL) {
+                        DEBUG(SSSDBG_MINOR_FAILURE, ("No SID for group [%s] " \
+                                                     "while id-mapping.\n",
+                                                     groupname));
+                        ret = EINVAL;
+                        goto done;
+                    }
+
+                    DEBUG(SSSDBG_TRACE_LIBS,
+                          ("Mapping group [%s] objectSID to unix ID\n", groupname));
 
                     DEBUG(SSSDBG_TRACE_INTERNAL,
                           ("Group [%s] has objectSID [%s]\n",
-                           name, sid_str));
+                           groupname, sid_str));
 
                     /* Convert the SID into a UNIX group ID */
                     ret = sdap_idmap_sid_to_unix(opts->idmap_ctx, sid_str,
@@ -134,7 +155,7 @@ static errno_t sdap_add_incomplete_groups(struct sysdb_ctx *sysdb,
                     if (ret == EOK) {
                         DEBUG(SSSDBG_TRACE_INTERNAL,
                               ("Group [%s] has mapped gid [%lu]\n",
-                               name, (unsigned long)gid));
+                               groupname, (unsigned long)gid));
                     } else {
                         posix = false;
                         gid = 0;
@@ -142,7 +163,7 @@ static errno_t sdap_add_incomplete_groups(struct sysdb_ctx *sysdb,
                         DEBUG(SSSDBG_TRACE_INTERNAL,
                               ("Group [%s] cannot be mapped. "
                                "Treating as a non-POSIX group\n",
-                               name));
+                               groupname));
                     }
 
                 } else {
@@ -150,9 +171,11 @@ static errno_t sdap_add_incomplete_groups(struct sysdb_ctx *sysdb,
                                                    SYSDB_GIDNUM,
                                                    &gid);
                     if (ret == ENOENT || (ret == EOK && gid == 0)) {
-                        DEBUG(9, ("The group %s gid was %s\n",
-                                  name, ret == ENOENT ? "missing" : "zero"));
-                        DEBUG(8, ("Marking group %s as non-posix and setting GID=0!\n", name));
+                        DEBUG(SSSDBG_TRACE_LIBS, ("The group %s gid was %s\n",
+                              groupname, ret == ENOENT ? "missing" : "zero"));
+                        DEBUG(SSSDBG_TRACE_FUNC,
+                              ("Marking group %s as non-posix and setting GID=0!\n",
+                              groupname));
                         gid = 0;
                         posix = false;
                     } else if (ret) {
@@ -169,9 +192,11 @@ static errno_t sdap_add_incomplete_groups(struct sysdb_ctx *sysdb,
                     original_dn = NULL;
                 }
 
-                DEBUG(8, ("Adding fake group %s to sysdb\n", name));
-                ret = sysdb_add_incomplete_group(sysdb, domain, name, gid,
-                                                 original_dn, posix, now);
+                DEBUG(SSSDBG_TRACE_INTERNAL,
+                      ("Adding fake group %s to sysdb\n", groupname));
+                ret = sysdb_add_incomplete_group(sysdb, domain, groupname, gid,
+                                                 original_dn, sid_str, posix,
+                                                 now);
                 if (ret != EOK) {
                     goto done;
                 }
@@ -717,11 +742,9 @@ static struct tevent_req *sdap_initgr_nested_send(TALLOC_CTX *memctx,
     state->user = user;
     state->op = NULL;
 
-    ret = sysdb_attrs_primary_name(sysdb, user,
-                                   opts->user_map[SDAP_AT_USER_NAME].name,
-                                   &state->username);
+    ret = sdap_get_user_primary_name(memctx, opts, user, dom, &state->username);
     if (ret != EOK) {
-        DEBUG(1, ("User entry had no username\n"));
+        DEBUG(SSSDBG_CRIT_FAILURE, ("User entry had no username\n"));
         goto immediate;
     }
 
@@ -1278,9 +1301,7 @@ sdap_initgr_nested_get_membership_diff(TALLOC_CTX *mem_ctx,
     }
 
     /* Get direct sysdb parents */
-    ret = sysdb_attrs_primary_name(sysdb, group,
-                                   opts->group_map[SDAP_AT_GROUP_NAME].name,
-                                   &group_name);
+    ret = sdap_get_group_primary_name(tmp_ctx, opts, group, dom, &group_name);
     if (ret != EOK) {
         goto done;
     }
@@ -1972,6 +1993,8 @@ errno_t save_rfc2307bis_user_memberships(
     char **add_groups;
     char **del_groups;
     bool in_transaction = false;
+    size_t c;
+    char *tmp_str;
 
     TALLOC_CTX *tmp_ctx = talloc_new(NULL);
     if(!tmp_ctx) {
@@ -2007,6 +2030,20 @@ errno_t save_rfc2307bis_user_memberships(
         if (ret != EOK) {
             goto error;
         }
+
+        if (IS_SUBDOMAIN(state->dom)) {
+            for (c = 0; ldap_grouplist[c] != NULL; c++) {
+                tmp_str = sss_tc_fqname(ldap_grouplist, state->dom->names,
+                                        state->dom, ldap_grouplist[c]);
+                if (tmp_str == NULL) {
+                    DEBUG(SSSDBG_OP_FAILURE, ("sss_tc_fqname failed.\n"));
+                    ret = ENOMEM;
+                    goto error;
+                }
+                talloc_free(ldap_grouplist[c]);
+                ldap_grouplist[c] = tmp_str;
+            }
+        }
     }
 
     /* Find the differences between the sysdb and ldap lists
@@ -2183,11 +2220,9 @@ static errno_t rfc2307bis_nested_groups_step(struct tevent_req *req)
         goto done;
     }
 
-    ret = sysdb_attrs_primary_name(
-            state->sysdb,
-            state->groups[state->group_iter],
-            state->opts->group_map[SDAP_AT_GROUP_NAME].name,
-            &state->primary_name);
+    ret = sdap_get_group_primary_name(state, state->opts,
+                                      state->groups[state->group_iter],
+                                      state->dom, &state->primary_name);
     if (ret != EOK) {
         goto done;
     }
@@ -2522,6 +2557,8 @@ struct sdap_get_initgr_state {
 
     size_t user_base_iter;
     struct sdap_search_base **user_search_bases;
+
+    bool use_id_mapping;
 };
 
 static errno_t sdap_get_initgr_next_base(struct tevent_req *req);
@@ -2590,6 +2627,10 @@ struct tevent_req *sdap_get_initgr_send(TALLOC_CTX *memctx,
         return NULL;
     }
 
+    state->use_id_mapping = sdap_idmap_domain_has_algorithmic_mapping(
+                                                         state->opts->idmap_ctx,
+                                                         state->dom->domain_id);
+
     ret = sdap_get_initgr_next_base(req);
 
 done:
@@ -2649,8 +2690,6 @@ static void sdap_get_initgr_user(struct tevent_req *subreq)
     const char *orig_dn;
     const char *cname;
     bool in_transaction = false;
-    bool use_id_mapping =
-            dp_opt_get_bool(state->opts->basic, SDAP_ID_MAPPING);
 
     DEBUG(9, ("Receiving info for the user\n"));
 
@@ -2753,7 +2792,7 @@ static void sdap_get_initgr_user(struct tevent_req *subreq)
             return;
         }
 
-        if (use_id_mapping
+        if (state->use_id_mapping
                 && state->opts->dc_functional_level >= DS_BEHAVIOR_WIN2008) {
             /* Take advantage of AD's tokenGroups mechanism to look up all
              * parent groups in a single request.
@@ -2835,7 +2874,6 @@ static void sdap_get_initgr_done(struct tevent_req *subreq)
     char *dom_sid_str;
     char *group_sid_str;
     struct sdap_options *opts = state->opts;
-    bool use_id_mapping = dp_opt_get_bool(opts->basic, SDAP_ID_MAPPING);
 
     DEBUG(9, ("Initgroups done\n"));
 
@@ -2852,7 +2890,7 @@ static void sdap_get_initgr_done(struct tevent_req *subreq)
 
     case SDAP_SCHEMA_RFC2307BIS:
     case SDAP_SCHEMA_AD:
-        if (use_id_mapping
+        if (state->use_id_mapping
                 && state->opts->dc_functional_level >= DS_BEHAVIOR_WIN2008) {
             ret = sdap_get_ad_tokengroups_initgroups_recv(subreq);
         }
@@ -2886,7 +2924,7 @@ static void sdap_get_initgr_done(struct tevent_req *subreq)
      * the user may not be an explicit member of that group
      */
 
-    if (use_id_mapping) {
+    if (state->use_id_mapping) {
         DEBUG(SSSDBG_TRACE_LIBS,
               ("Mapping primary group to unix ID\n"));
 
@@ -2934,7 +2972,8 @@ static void sdap_get_initgr_done(struct tevent_req *subreq)
         }
 
         /* Convert the SID into a UNIX group ID */
-        ret = sdap_idmap_sid_to_unix(opts->idmap_ctx, sid_str, &primary_gid);
+        ret = sdap_idmap_sid_to_unix(opts->idmap_ctx, group_sid_str,
+                                     &primary_gid);
         if (ret != EOK) goto fail;
     } else {
         ret = sysdb_attrs_get_uint32_t(state->orig_user, SYSDB_GIDNUM,
