@@ -98,6 +98,7 @@ int sdap_save_users(TALLOC_CTX *memctx,
                     char **_usn_value);
 
 int sdap_initgr_common_store(struct sysdb_ctx *sysdb,
+                             struct sss_domain_info *domain,
                              struct sdap_options *opts,
                              const char *name,
                              enum sysdb_member_type type,
@@ -107,7 +108,23 @@ int sdap_initgr_common_store(struct sysdb_ctx *sysdb,
 
 errno_t get_sysdb_grouplist(TALLOC_CTX *mem_ctx,
                             struct sysdb_ctx *sysdb,
+                            struct sss_domain_info *domain,
                             const char *name,
                             char ***grouplist);
 
+/* from sdap_async_nested_groups.c */
+struct tevent_req *sdap_nested_group_send(TALLOC_CTX *mem_ctx,
+                                          struct tevent_context *ev,
+                                          struct sdap_domain *sdom,
+                                          struct sdap_options *opts,
+                                          struct sdap_handle *sh,
+                                          struct sysdb_attrs *group);
+
+errno_t sdap_nested_group_recv(TALLOC_CTX *mem_ctx,
+                               struct tevent_req *req,
+                               unsigned long *_num_users,
+                               struct sysdb_attrs ***_users,
+                               unsigned long *_num_groups,
+                               struct sysdb_attrs ***_groups);
+
 #endif /* _SDAP_ASYNC_PRIVATE_H_ */
