@@ -475,7 +475,7 @@ sdap_save_service(TALLOC_CTX *mem_ctx,
 
     cache_timeout = dom->service_timeout;
 
-    ret = sysdb_store_service(sysdb, name, port, aliases, store_protocols,
+    ret = sysdb_store_service(sysdb, dom, name, port, aliases, store_protocols,
                               svc_attrs, missing, cache_timeout, now);
     if (ret != EOK) {
         DEBUG(SSSDBG_MINOR_FAILURE,
@@ -543,7 +543,7 @@ enum_services_send(TALLOC_CTX *memctx,
     state->ev = ev;
     state->id_ctx = id_ctx;
     state->domain = id_ctx->be->domain;
-    state->sysdb = id_ctx->be->sysdb;
+    state->sysdb = id_ctx->be->domain->sysdb;
     state->op = op;
 
     if (id_ctx->srv_opts && id_ctx->srv_opts->max_service_value && !purge) {
@@ -582,7 +582,7 @@ enum_services_send(TALLOC_CTX *memctx,
     subreq = sdap_get_services_send(state, state->ev,
                                     state->domain, state->sysdb,
                                     state->id_ctx->opts,
-                                    state->id_ctx->opts->service_search_bases,
+                                    state->id_ctx->opts->sdom->service_search_bases,
                                     sdap_id_op_handle(state->op),
                                     state->attrs, state->filter,
                                     dp_opt_get_int(state->id_ctx->opts->basic,
