@@ -37,6 +37,82 @@ sdap_idmap_talloc_free(void *ptr, void *pvt)
     talloc_free(ptr);
 }
 
+static errno_t
+sdap_idmap_add_configured_external_range(struct sdap_idmap_ctx *idmap_ctx)
+{
+    int int_id;
+    struct sss_idmap_range range;
+    struct sdap_id_ctx *id_ctx;
+    enum idmap_error_code err;
+
+    if (idmap_ctx == NULL) {
+        return EINVAL;
+    }
+
+    id_ctx = idmap_ctx->id_ctx;
+
+    int_id = dp_opt_get_int(id_ctx->opts->basic, SDAP_MIN_ID);
+    if (int_id < 0) {
+        DEBUG(SSSDBG_CONF_SETTINGS, ("ldap_min_id must be greater than 0.\n"));
+        return EINVAL;
+    }
+    range.min = int_id;
+
+    int_id = dp_opt_get_int(id_ctx->opts->basic, SDAP_MAX_ID);
+    if (int_id < 0) {
+        DEBUG(SSSDBG_CONF_SETTINGS, ("ldap_min_id must be greater than 0.\n"));
+        return EINVAL;
+    }
+    range.max = int_id;
+
+    if ((range.min == 0 && range.max != 0)
+            || (range.min != 0 && range.max == 0)) {
+        DEBUG(SSSDBG_CONF_SETTINGS, ("Both ldap_min_id and ldap_max_id " \
+                                     "either must be 0 (not set) " \
+                                     "or positive integers.\n"));
+        return EINVAL;
+    }
+
+    if (range.min == 0 && range.max == 0) {
+        /* ldap_min_id and ldap_max_id not set, using min_id and max_id */
+        range.min = id_ctx->be->domain->id_min;
+        range.max = id_ctx->be->domain->id_max;
+        if (range.max == 0) {
+            range.max = UINT32_MAX;
+        }
+    }
+
+    err = sss_idmap_add_domain_ex(idmap_ctx->map, id_ctx->be->domain->name,
+                                  id_ctx->be->domain->domain_id, &range,
+                                  NULL, 0, true);
+    if (err != IDMAP_SUCCESS) {
+        DEBUG(SSSDBG_CRIT_FAILURE,
+              ("Could not add domain [%s] to the map: [%d]\n",
+               id_ctx->be->domain->name, err));
+        return EIO;
+    }
+
+    return EOK;
+}
+
+errno_t sdap_idmap_find_new_domain(struct sdap_idmap_ctx *idmap_ctx,
+                                   const char *dom_name,
+                                   const char *dom_sid_str)
+{
+    int ret;
+
+    ret = sdap_idmap_add_domain(idmap_ctx,
+                                dom_name, dom_sid_str,
+                                -1);
+    if (ret != EOK) {
+        DEBUG(SSSDBG_MINOR_FAILURE,
+              ("Could not add new domain [%s]\n", dom_name));
+        return ret;
+    }
+
+    return EOK;
+}
+
 errno_t
 sdap_idmap_init(TALLOC_CTX *mem_ctx,
                 struct sdap_id_ctx *id_ctx,
@@ -66,6 +142,7 @@ sdap_idmap_init(TALLOC_CTX *mem_ctx,
         goto done;
     }
     idmap_ctx->id_ctx = id_ctx;
+    idmap_ctx->find_new_domain = sdap_idmap_find_new_domain;
 
     idmap_lower = dp_opt_get_int(idmap_ctx->id_ctx->opts->basic,
                                  SDAP_IDMAP_LOWER);
@@ -120,6 +197,18 @@ sdap_idmap_init(TALLOC_CTX *mem_ctx,
         return EIO;
     }
 
+
+    /* Setup range for externally managed IDs, i.e. IDs are read from the
+     * ldap_user_uid_number and ldap_group_gid_number attributes. */
+    if (!dp_opt_get_bool(idmap_ctx->id_ctx->opts->basic, SDAP_ID_MAPPING)) {
+        ret = sdap_idmap_add_configured_external_range(idmap_ctx);
+        if (ret != EOK) {
+            DEBUG(SSSDBG_OP_FAILURE,
+                  ("sdap_idmap_add_configured_external_range failed.\n"));
+            goto done;
+        }
+    }
+
     /* Read in any existing mappings from the cache */
     ret = sysdb_idmap_get_mappings(tmp_ctx, sysdb, id_ctx->be->domain, &res);
     if (ret != EOK && ret != ENOENT) {
@@ -348,9 +437,7 @@ sdap_idmap_sid_to_unix(struct sdap_idmap_ctx *idmap_ctx,
             goto done;
         }
 
-        ret = sdap_idmap_add_domain(idmap_ctx,
-                                    dom_sid_str, dom_sid_str,
-                                    -1);
+        ret = idmap_ctx->find_new_domain(idmap_ctx, dom_sid_str, dom_sid_str);
         if (ret != EOK) {
             DEBUG(SSSDBG_MINOR_FAILURE,
                   ("Could not add new domain for sid [%s]\n", sid_str));
@@ -390,3 +477,62 @@ done:
     talloc_free(dom_sid_str);
     return ret;
 }
+
+bool sdap_idmap_domain_has_algorithmic_mapping(struct sdap_idmap_ctx *ctx,
+                                               const char *dom_sid)
+{
+    enum idmap_error_code err;
+    bool has_algorithmic_mapping;
+    char *new_dom_sid;
+    int ret;
+    TALLOC_CTX *tmp_ctx = NULL;
+
+    err = sss_idmap_domain_has_algorithmic_mapping(ctx->map, dom_sid,
+                                                   &has_algorithmic_mapping);
+    if (err == IDMAP_SUCCESS) {
+        return has_algorithmic_mapping;
+    } else if (err != IDMAP_SID_UNKNOWN && err != IDMAP_NO_DOMAIN) {
+        return false;
+    }
+
+    /* This is the first time we've seen this domain
+     * Create a new domain for it. We'll use the dom-sid
+     * as the domain name for now, since we don't have
+     * any way to get the real name.
+     */
+
+    if (is_domain_sid(dom_sid)) {
+        new_dom_sid = discard_const(dom_sid);
+    } else {
+        tmp_ctx = talloc_new(NULL);
+        if (tmp_ctx == NULL) {
+            DEBUG(SSSDBG_OP_FAILURE, ("talloc_new failed.\n"));
+            return false;
+        }
+
+        ret = sdap_idmap_get_dom_sid_from_object(tmp_ctx, dom_sid,
+                                                 &new_dom_sid);
+        if (ret != EOK) {
+            DEBUG(SSSDBG_MINOR_FAILURE,
+                  ("Could not parse domain SID from [%s]\n", dom_sid));
+            talloc_free(tmp_ctx);
+            return false;
+        }
+    }
+
+    ret = ctx->find_new_domain(ctx, new_dom_sid, new_dom_sid);
+    talloc_free(tmp_ctx);
+    if (ret != EOK) {
+        DEBUG(SSSDBG_MINOR_FAILURE,
+              ("Could not add new domain for sid [%s]\n", dom_sid));
+        return false;
+    }
+
+    err = sss_idmap_domain_has_algorithmic_mapping(ctx->map, dom_sid,
+                                                   &has_algorithmic_mapping);
+    if (err == IDMAP_SUCCESS) {
+        return has_algorithmic_mapping;
+    }
+
+    return false;
+}
