@@ -43,13 +43,14 @@ struct sbus_interface proxy_interface = {
 static void proxy_shutdown(struct be_req *req)
 {
     /* TODO: Clean up any internal data */
-    req->fn(req, DP_ERR_OK, EOK, NULL);
+    be_req_terminate(req, DP_ERR_OK, EOK, NULL);
 }
 
 static void proxy_auth_shutdown(struct be_req *req)
 {
-    talloc_free(req->be_ctx->bet_info[BET_AUTH].pvt_bet_data);
-    req->fn(req, DP_ERR_OK, EOK, NULL);
+    struct be_ctx *be_ctx = be_req_get_be_ctx(req);
+    talloc_free(be_ctx->bet_info[BET_AUTH].pvt_bet_data);
+    be_req_terminate(req, DP_ERR_OK, EOK, NULL);
 }
 
 struct bet_ops proxy_id_ops = {
@@ -109,7 +110,7 @@ int sssm_proxy_id_init(struct be_ctx *bectx,
         goto done;
     }
 
-    ret = confdb_get_bool(bectx->cdb, ctx, bectx->conf_path,
+    ret = confdb_get_bool(bectx->cdb, bectx->conf_path,
                           CONFDB_PROXY_FAST_ALIAS, false, &ctx->fast_alias);
     if (ret != EOK) goto done;
 
