@@ -35,6 +35,7 @@
 #include "responder/nss/nsssrv.h"
 #include "responder/nss/nsssrv_private.h"
 #include "responder/nss/nsssrv_mmap_cache.h"
+#include "responder/nss/nsssrv_netgroup.h"
 #include "responder/common/negcache.h"
 #include "db/sysdb.h"
 #include "confdb/confdb.h"
@@ -55,12 +56,15 @@
 
 static int nss_clear_memcache(DBusMessage *message,
                               struct sbus_connection *conn);
+static int nss_clear_netgroup_hash_table(DBusMessage *message,
+                                         struct sbus_connection *conn);
 
 struct sbus_method monitor_nss_methods[] = {
     { MON_CLI_METHOD_PING, monitor_common_pong },
     { MON_CLI_METHOD_RES_INIT, monitor_common_res_init },
     { MON_CLI_METHOD_ROTATE, responder_logrotate },
     { MON_CLI_METHOD_CLEAR_MEMCACHE, nss_clear_memcache},
+    { MON_CLI_METHOD_CLEAR_ENUM_CACHE, nss_clear_netgroup_hash_table},
     { NULL, NULL }
 };
 
@@ -131,6 +135,24 @@ done:
     return monitor_common_pong(message, conn);
 }
 
+static int nss_clear_netgroup_hash_table(DBusMessage *message,
+                                         struct sbus_connection *conn)
+{
+    errno_t ret;
+    struct resp_ctx *rctx = talloc_get_type(sbus_conn_get_private_data(conn),
+                                            struct resp_ctx);
+    struct nss_ctx *nctx = (struct nss_ctx*) rctx->pvt_ctx;
+
+    ret = nss_orphan_netgroups(nctx);
+    if (ret != EOK) {
+        DEBUG(SSSDBG_CRIT_FAILURE,
+              ("Could not invalidate netgroups\n"));
+        return ret;
+    }
+
+    return monitor_common_pong(message, conn);
+}
+
 static errno_t nss_get_etc_shells(TALLOC_CTX *mem_ctx, char ***_shells)
 {
     int i = 0;
@@ -477,7 +499,8 @@ int nss_process_init(TALLOC_CTX *mem_ctx,
     }
 
     /* Create the lookup table for netgroup results */
-    hret = sss_hash_create(nctx, 10, &nctx->netgroups);
+    hret = sss_hash_create_ex(nctx, 10, &nctx->netgroups, 0, 0, 0, 0,
+                              netgroup_hash_delete_cb, NULL);
     if (hret != HASH_SUCCESS) {
         DEBUG(0,("Unable to initialize netgroup hash table\n"));
         ret = EIO;
