@@ -44,7 +44,7 @@ enum pam_verbosity {
 
 static void pam_reply(struct pam_auth_req *preq);
 
-static int extract_authtok_v2(TALLOC_CTX *mem_ctx, struct sss_auth_token *tok,
+static int extract_authtok_v2(struct sss_auth_token *tok,
                               size_t data_size, uint8_t *body, size_t blen,
                               size_t *c)
 {
@@ -65,8 +65,12 @@ static int extract_authtok_v2(TALLOC_CTX *mem_ctx, struct sss_auth_token *tok,
         sss_authtok_set_empty(tok);
         break;
     case SSS_AUTHTOK_TYPE_PASSWORD:
-        ret = sss_authtok_set_password(tok, (const char *)auth_token_data,
-                                       auth_token_length);
+        if (auth_token_length == 0) {
+            sss_authtok_set_empty(tok);
+        } else {
+            ret = sss_authtok_set_password(tok, (const char *)auth_token_data,
+                                           auth_token_length);
+        }
         break;
     default:
         return EINVAL;
@@ -196,12 +200,12 @@ static int pam_parse_in_data_v2(struct sss_domain_info *domains,
                     if (ret != EOK) return ret;
                     break;
                 case SSS_PAM_ITEM_AUTHTOK:
-                    ret = extract_authtok_v2(pd, pd->authtok,
+                    ret = extract_authtok_v2(pd->authtok,
                                              size, body, blen, &c);
                     if (ret != EOK) return ret;
                     break;
                 case SSS_PAM_ITEM_NEWAUTHTOK:
-                    ret = extract_authtok_v2(pd, pd->newauthtok,
+                    ret = extract_authtok_v2(pd->newauthtok,
                                              size, body, blen, &c);
                     if (ret != EOK) return ret;
                     break;
@@ -242,7 +246,7 @@ static int pam_parse_in_data_v3(struct sss_domain_info *domains,
     return EOK;
 }
 
-static int extract_authtok_v1(TALLOC_CTX *mem_ctx, struct sss_auth_token *tok,
+static int extract_authtok_v1(struct sss_auth_token *tok,
                               uint8_t *body, size_t blen, size_t *c)
 {
     uint32_t auth_token_type;
@@ -308,12 +312,12 @@ static int pam_parse_in_data(struct sss_domain_info *domains,
     if (body[end++] != '\0') return EINVAL;
     pd->rhost = (char *) &body[start];
 
-    ret = extract_authtok_v1(pd, pd->authtok, body, blen, &end);
+    ret = extract_authtok_v1(pd->authtok, body, blen, &end);
     if (ret) {
         DEBUG(1, ("Invalid auth token\n"));
         return ret;
     }
-    ret = extract_authtok_v1(pd, pd->newauthtok, body, blen, &end);
+    ret = extract_authtok_v1(pd->newauthtok, body, blen, &end);
     if (ret) {
         DEBUG(1, ("Invalid new auth token\n"));
         return ret;
@@ -802,6 +806,14 @@ static int pam_forwarder(struct cli_ctx *cctx, int pam_cmd)
             ret = ENOENT;
             goto done;
         }
+
+        ncret = sss_ncache_check_user(pctx->ncache, pctx->neg_timeout,
+                                      preq->domain, pd->user);
+        if (ncret == EEXIST) {
+            /* User found in the negative cache */
+            ret = ENOENT;
+            goto done;
+        }
     } else {
         for (dom = preq->cctx->rctx->domains;
              dom;
@@ -963,6 +975,17 @@ static int pam_check_user_search(struct pam_auth_req *preq)
         }
 
         if (preq->res->count == 0) {
+            if (preq->check_provider == false) {
+                /* set negative cache only if not result of cache check */
+                ret = sss_ncache_set_user(pctx->ncache, false, dom, name);
+                if (ret != EOK) {
+                    /* Should not be fatal, just slower next time */
+                    DEBUG(SSSDBG_MINOR_FAILURE,
+                           ("Cannot set ncache for [%s@%s]\n", name,
+                            dom->name));
+                }
+            }
+
             /* if a multidomain search, try with next */
             if (!preq->pd->domain) {
                 dom = get_next_domain(dom, false);
