@@ -53,13 +53,13 @@ static int setup_db(struct tools_ctx *ctx)
         return ret;
     }
 
-    ret = sysdb_init_domain_and_sysdb(ctx, ctx->confdb, "local", DB_PATH,
-                                      &ctx->local, &ctx->sysdb);
+    ret = sssd_domain_init(ctx, ctx->confdb, "local", DB_PATH, &ctx->local);
     if (ret != EOK) {
         SYSDB_VERSION_ERROR(ret);
         DEBUG(1, ("Could not initialize connection to the sysdb\n"));
         return ret;
     }
+    ctx->sysdb = ctx->local->sysdb;
 
     talloc_free(confdb_path);
     return EOK;
@@ -70,8 +70,20 @@ static int setup_db(struct tools_ctx *ctx)
  */
 void usage(poptContext pc, const char *error)
 {
+    size_t lentmp;
+    char nl[2] = "";
+
     poptPrintUsage(pc, stderr, 0);
-    if (error) fprintf(stderr, "%s", error);
+
+    if (error) {
+        lentmp = strlen(error);
+        if ((lentmp > 0) && (error[lentmp - 1] != '\n')) {
+            nl[0]='\n';
+            nl[1]='\0';
+        }
+
+        fprintf(stderr, "%s%s", error, nl);
+    }
 }
 
 int parse_groups(TALLOC_CTX *mem_ctx, const char *optstr, char ***_out)
@@ -180,6 +192,15 @@ int parse_name_domain(struct tools_ctx *tctx,
             DEBUG(1, ("Invalid domain %s specified in FQDN\n", domain));
             return EINVAL;
         }
+    } else {
+        if (tctx->local->fqnames) {
+            DEBUG(SSSDBG_CRIT_FAILURE,
+                  ("Name '%s' does not seem to be FQDN "
+                   "('%s = TRUE' is set)\n", fullname, CONFDB_DOMAIN_FQ));
+            ERROR("Name '%1$s' does not seem to be FQDN "
+                  "('%2$s = TRUE' is set)\n", fullname, CONFDB_DOMAIN_FQ);
+            return EINVAL;
+        }
     }
 
     return EOK;
@@ -197,6 +218,7 @@ int check_group_names(struct tools_ctx *tctx,
     if (!groupinfo) {
         return ENOMEM;
     }
+    groupinfo->domain = tctx->local;
 
     ret = EOK;
     for (i=0; grouplist[i]; ++i) {
@@ -510,18 +532,13 @@ int run_userdel_cmd(struct tools_ctx *tctx)
     } else {
         /* parent */
         if (pid == -1) {
-            DEBUG(1, ("fork failed [%d]: %s\n"));
             ret = errno;
+            DEBUG(SSSDBG_CRIT_FAILURE,
+                  ("fork failed [%d]: %s\n", ret, strerror(ret)));
             goto done;
         }
 
         while((child_pid = waitpid(pid, &status, 0)) > 0) {
-            if (child_pid == -1) {
-                DEBUG(1, ("waitpid failed\n"));
-                ret = errno;
-                goto done;
-            }
-
             if (WIFEXITED(status)) {
                 ret = WEXITSTATUS(status);
                 if (ret != 0) {
@@ -545,6 +562,11 @@ int run_userdel_cmd(struct tools_ctx *tctx)
                 goto done;
             }
         }
+        if (child_pid == -1) {
+            DEBUG(SSSDBG_CRIT_FAILURE, ("waitpid failed\n"));
+            ret = errno;
+            goto done;
+        }
     }
 
     ret = EOK;
