@@ -262,7 +262,8 @@ struct sss_child_ctx_old {
 };
 
 int child_handler_setup(struct tevent_context *ev, int pid,
-                        sss_child_callback_t cb, void *pvt)
+                        sss_child_callback_t cb, void *pvt,
+                        struct sss_child_ctx_old **_child_ctx)
 {
     struct sss_child_ctx_old *child_ctx;
 
@@ -286,9 +287,30 @@ int child_handler_setup(struct tevent_context *ev, int pid,
     child_ctx->pvt = pvt;
 
     DEBUG(8, ("Signal handler set up for pid [%d]\n", pid));
+
+    if (_child_ctx != NULL) {
+        *_child_ctx = child_ctx;
+    }
+
     return EOK;
 }
 
+void child_handler_destroy(struct sss_child_ctx_old *ctx)
+{
+    errno_t ret;
+
+    /* We still want to wait for the child to finish, but the caller is not
+     * interested in the result anymore (e.g. timeout was reached). */
+    ctx->cb = NULL;
+    ctx->pvt = NULL;
+
+    ret = kill(ctx->pid, SIGKILL);
+    if (ret == -1) {
+        ret = errno;
+        DEBUG(SSSDBG_MINOR_FAILURE, ("kill failed [%d][%s].\n", ret, strerror(ret)));
+    }
+}
+
 /* Async communication with the child process via a pipe */
 
 struct write_pipe_state {
@@ -531,22 +553,22 @@ void child_sig_handler(struct tevent_context *ev,
     } else if (ret == 0) {
         DEBUG(1, ("waitpid did not found a child with changed status.\n"));
     } else {
-        if WIFEXITED(child_ctx->child_status) {
+        if (WIFEXITED(child_ctx->child_status)) {
             if (WEXITSTATUS(child_ctx->child_status) != 0) {
                 DEBUG(1, ("child [%d] failed with status [%d].\n", ret,
                           WEXITSTATUS(child_ctx->child_status)));
             } else {
                 DEBUG(4, ("child [%d] finished successfully.\n", ret));
             }
-        } else if WIFSIGNALED(child_ctx->child_status) {
+        } else if (WIFSIGNALED(child_ctx->child_status)) {
             DEBUG(1, ("child [%d] was terminated by signal [%d].\n", ret,
                       WTERMSIG(child_ctx->child_status)));
         } else {
-            if WIFSTOPPED(child_ctx->child_status) {
+            if (WIFSTOPPED(child_ctx->child_status)) {
                 DEBUG(7, ("child [%d] was stopped by signal [%d].\n", ret,
                           WSTOPSIG(child_ctx->child_status)));
             }
-            if WIFCONTINUED(child_ctx->child_status) {
+            if (WIFCONTINUED(child_ctx->child_status)) {
                 DEBUG(7, ("child [%d] was resumed by delivery of SIGCONT.\n",
                           ret));
             }
