@@ -221,6 +221,7 @@ int pidfile(const char *path, const char *name)
         err = errno;
         DEBUG(SSSDBG_CRIT_FAILURE,
               ("write failed [%d][%s]\n", err, strerror(err)));
+        close(fd);
         return err;
     }
 
@@ -316,7 +317,7 @@ static void setup_signals(void)
 #endif
 
     /* POSIX demands that signals are inherited. If the invoking process has
-     * these signals masked, we will have problems, as we won't recieve them. */
+     * these signals masked, we will have problems, as we won't receive them. */
     BlockSignals(false, SIGHUP);
     BlockSignals(false, SIGTERM);
 
