@@ -485,6 +485,7 @@ const char *KRB5_CALLCONV sss_krb5_get_error_message(krb5_context ctx,
     ret = snprintf(s, size, "Kerberos error [%12d]", ec);
 
     if (ret < 0 || ret >= size) {
+        free(s);
         return NULL;
     }
 
@@ -640,7 +641,7 @@ errno_t check_fast(const char *str, bool *use_fast)
         *use_fast = true;
     } else {
         sss_log(SSS_LOG_ALERT, "Unsupported value [%s] for option krb5_use_fast,"
-                               "please use never, try, or demand.\n");
+                               "please use never, try, or demand.\n", str);
         return EINVAL;
     }
 
@@ -927,6 +928,7 @@ sss_krb5_free_keytab_entry_contents(krb5_context context,
 
 #define SSS_KRB5_FILE   "FILE:"
 #define SSS_KRB5_DIR    "DIR:"
+#define SSS_KRB5_KEYRING "KEYRING:"
 
 enum sss_krb5_cc_type
 sss_krb5_get_type(const char *full_location)
@@ -939,12 +941,17 @@ sss_krb5_get_type(const char *full_location)
                 sizeof(SSS_KRB5_FILE)-1) == 0) {
         return SSS_KRB5_TYPE_FILE;
     }
-#ifdef HAVE_KRB5_DIRCACHE
+#ifdef HAVE_KRB5_CC_COLLECTION
     else if (strncmp(full_location, SSS_KRB5_DIR,
                sizeof(SSS_KRB5_DIR)-1) == 0) {
         return SSS_KRB5_TYPE_DIR;
     }
-#endif /* HAVE_KRB5_DIRCACHE */
+    else if (strncmp(full_location, SSS_KRB5_KEYRING,
+               sizeof(SSS_KRB5_KEYRING)-1) == 0) {
+        return SSS_KRB5_TYPE_KEYRING;
+    }
+#endif /* HAVE_KRB5_CC_COLLECTION */
+
     else if (full_location[0] == '/') {
         return SSS_KRB5_TYPE_FILE;
     }
@@ -968,11 +975,16 @@ sss_krb5_residual_by_type(const char *full_location,
                 offset = sizeof(SSS_KRB5_FILE)-1;
             }
             break;
-#ifdef HAVE_KRB5_DIRCACHE
+#ifdef HAVE_KRB5_CC_COLLECTION
         case SSS_KRB5_TYPE_DIR:
             offset = sizeof(SSS_KRB5_DIR)-1;
             break;
-#endif /* HAVE_KRB5_DIRCACHE */
+
+        case SSS_KRB5_TYPE_KEYRING:
+            offset = sizeof(SSS_KRB5_KEYRING)-1;
+            break;
+#endif /* HAVE_KRB5_CC_COLLECTION */
+
         default:
             return NULL;
     }
@@ -990,16 +1002,19 @@ sss_krb5_cc_file_path(const char *full_location)
     residual = sss_krb5_residual_by_type(full_location, cc_type);
 
     switch(cc_type) {
+#ifdef HAVE_KRB5_CC_COLLECTION
+        case SSS_KRB5_TYPE_KEYRING:
+#endif /* HAVE_KRB5_CC_COLLECTION */
         case SSS_KRB5_TYPE_FILE:
             return residual;
-#ifdef HAVE_KRB5_DIRCACHE
+#ifdef HAVE_KRB5_CC_COLLECTION
         case SSS_KRB5_TYPE_DIR:
             /* DIR::/run/user/tkt_foo */
             if (residual[0] == ':') {
                 ++residual;
             }
             return residual;
-#endif
+#endif /* HAVE_KRB5_CC_COLLECTION */
         case SSS_KRB5_TYPE_UNKNOWN:
             break;
     }
@@ -1179,3 +1194,61 @@ done:
     return ENOTSUP;
 #endif
 }
+
+char * sss_get_ccache_name_for_principal(TALLOC_CTX *mem_ctx,
+                                         krb5_context ctx,
+                                         krb5_principal principal,
+                                         const char *location)
+{
+#ifdef HAVE_KRB5_CC_COLLECTION
+    krb5_error_code kerr;
+    krb5_ccache tmp_cc = NULL;
+    char *tmp_ccname = NULL;
+    char *ret_ccname = NULL;
+
+    DEBUG(SSSDBG_TRACE_ALL,
+          ("Location: [%s]\n", location));
+
+    kerr = krb5_cc_set_default_name(ctx, location);
+    if (kerr != 0) {
+        KRB5_DEBUG(SSSDBG_MINOR_FAILURE, ctx, kerr);
+        return NULL;
+    }
+
+    kerr = krb5_cc_cache_match(ctx, principal, &tmp_cc);
+    if (kerr != 0) {
+        const char *err_msg = sss_krb5_get_error_message(ctx, kerr);
+        DEBUG(SSSDBG_TRACE_INTERNAL,
+              ("krb5_cc_cache_match failed: [%d][%s]\n", kerr, err_msg));
+        sss_krb5_free_error_message(ctx, err_msg);
+        return NULL;
+    }
+
+    kerr = krb5_cc_get_full_name(ctx, tmp_cc, &tmp_ccname);
+    if (kerr != 0) {
+        KRB5_DEBUG(SSSDBG_MINOR_FAILURE, ctx, kerr);
+        goto done;
+    }
+
+    DEBUG(SSSDBG_TRACE_ALL,
+          ("tmp_ccname: [%s]\n", tmp_ccname));
+
+    ret_ccname = talloc_strdup(mem_ctx, tmp_ccname);
+    if (ret_ccname == NULL) {
+        DEBUG(SSSDBG_OP_FAILURE, ("talloc_strdup failed (ENOMEM).\n"));
+    }
+
+done:
+    if (tmp_cc != NULL) {
+        kerr = krb5_cc_close(ctx, tmp_cc);
+        if (kerr != 0) {
+            KRB5_DEBUG(SSSDBG_MINOR_FAILURE, ctx, kerr);
+        }
+    }
+    krb5_free_string(ctx, tmp_ccname);
+
+    return ret_ccname;
+#else
+    return NULL;
+#endif /* HAVE_KRB5_CC_COLLECTION */
+}
