@@ -238,25 +238,6 @@ errno_t set_debug_file_from_fd(const int fd);
 #endif
 #endif
 
-/* TODO: remove later
- * These functions are available in the latest tevent and are the ones that
- * should be used as tevent_req is rightfully opaque there */
-#ifndef tevent_req_data
-#define tevent_req_data(req, type) ((type *)req->private_state)
-#define tevent_req_set_callback(req, func, data) \
-    do { req->async.fn = func; req->async.private_data = data; } while(0)
-#define tevent_req_callback_data(req, type) ((type *)req->async.private_data)
-#define tevent_req_notify_callback(req) \
-    do { \
-        if (req->async.fn != NULL) { \
-            req->async.fn(req); \
-        } \
-    } while(0)
-
-/* noop */
-#define tevent_loop_allow_nesting(x)
-#endif
-
 #define TEVENT_REQ_RETURN_ON_ERROR(req) do { \
     enum tevent_req_state TRROEstate; \
     uint64_t TRROEerr; \
@@ -550,19 +531,23 @@ struct sized_string {
 
 void to_sized_string(struct sized_string *out, const char *in);
 
-/* form domain_info.c */
+/* from domain_info.c */
 struct sss_domain_info *get_next_domain(struct sss_domain_info *domain,
                                         bool descend);
 struct sss_domain_info *find_subdomain_by_name(struct sss_domain_info *domain,
                                                const char *name,
                                                bool match_any);
+bool subdomain_enumerates(struct sss_domain_info *parent,
+                          const char *sd_name);
 
 struct sss_domain_info *new_subdomain(TALLOC_CTX *mem_ctx,
                                       struct sss_domain_info *parent,
                                       const char *name,
                                       const char *realm,
                                       const char *flat_name,
-                                      const char *id);
+                                      const char *id,
+                                      bool mpg,
+                                      bool enumerate);
 
 errno_t sssd_domain_init(TALLOC_CTX *mem_ctx,
                          struct confdb_ctx *cdb,
@@ -579,25 +564,6 @@ errno_t sss_br_lock_file(int fd, size_t start, size_t len,
                          int num_tries, useconds_t wait);
 #include "io.h"
 
-/* Endianness-compatibility for systems running older versions of glibc */
-
-#ifndef le32toh
-#include <byteswap.h>
-
-/* Copied from endian.h on glibc 2.15 */
-#ifdef __USE_BSD
-/* Conversion interfaces.  */
-# if __BYTE_ORDER == __LITTLE_ENDIAN
-#  define le32toh(x) (x)
-#  define htole32(x) (x)
-# else
-#  define le32toh(x) __bswap_32 (x)
-#  define htole32(x) __bswap_32 (x)
-# endif
-#endif /* __USE_BSD */
-
-#endif /* le32toh */
-
 #ifdef HAVE_PAC_RESPONDER
 #define BUILD_WITH_PAC_RESPONDER true
 #else
